#Scenario 2
##treatment train: NF-> MED-> THERMAL CRYST                 
from desalsim.nanofiltration_unit_f import OsmoticPressure
from desalsim.nanofiltration_unit_f import NFMass
from desalsim.nanofiltration_unit_f import NfEnergy

from desalsim.med_unit_f import MEDCalculator

from desalsim.thermal_cryst_f import thermal_calc
from desalsim.thermal_cryst_f import conc_cal
from desalsim.thermal_cryst_f import calculate_energy


from desalsim import constants

from desalsim.economic_f import revenue
from desalsim.economic_f import econom

from desalsim import scaleup
from desalsim.density_calc import density_calc
import numpy as np
import pandas as pd
#%%
#Molecular weight 
    #molecular weight
MW_Na=constants.MW_Na
MW_Cl=constants.MW_cl
MW_SO4=constants.MW_so4
MW_K=constants.MW_K
MW_Ca=constants.MW_Ca
MW_Mg=constants.MW_Mg
MW_HCO3=constants.MW_HCO3
MW_H=1.00784
MW_OH=17.008
MW_H2O=MW_H+MW_OH
MW_values = [MW_Na, MW_Cl, MW_K, MW_Mg, MW_Ca, MW_SO4]
MW_MgOH=58.3197
MW_CaOH=74.09
MW_Na2SO4=142.039458
MW_HCl=36.458
MW_NaOH=39.997
MW_NaCl=58.442729
MW_Na2SO4=142.04  
MW_Na2SO4_10h20=322.192258                                                                                                   #Molecular weight of sodium sulfate : g/mol
MW_KCl=74.5513
MW_CaSO4=constants.MW_Ca+constants.MW_so4

#constants 
d_Na2SO4=2.66*1000                                                                                                  #Density of sodium sulfate (25oC) : kg/m^3 (source:???) 
Cp_Na2SO4=128.2/MW_Na2SO4*1000   
d_NaCl=2.16*1000                                                                                                    #Density of NaCl (25oC) : kg/m^3 (source:https://pubchem.ncbi.nlm.nih.gov/compound/sodium_chloride#section=Solubility) 
Cp_NaCl=(50.5*MW_NaCl)*1000    
d_w=1*1000                                                                                                      #Density of water (25oC) : kg/m^3 (source:Wikipedia) 
Cp_w=4.1813*1000   
d_KCl=1.98*1000


#input data
#emissions
CO2_el=0.275 # kg/kwh
CO2_st=0#.255 # kg/kwh

#input data
T=20+273 #Operating temperature (units: K)
#Feed concentration
components = ['Na', 'Cl', 'K', 'Mg', 'Ca', 'SO4']
Ci_in = [12.33, 21.67, 0.45, 1.39, 0.45, 3.28]
z_values = [1, -1, 1, 2, 2, -2]
c_values = [Ci / 1000 for Ci in Ci_in]
mg_in = sum(c_values)
#Feed flow density 
d_in = density_calc(T-273, mg_in)  # kg/m3

#Feed flowrate
Qsw = 3000 / 24 * d_in #m3/d
#%%
#Calculations
"""--------Nanofiltration--------"""

Qf_nf = Qsw  # kg/hr
#Constants
R=8.314 #gas constant (units: J / mol·K)

#Asuumptions  
rjr_values = [0.16, 0.29, 0.21, 0.98, 0.95, 0.98] #Ions rejection rates based on membrane characteristics (units: -)
Wrec = 0.7 # Water recovery based on membrane characteristics (units: -)
n=0.8 #pump efficiency (units: -)
dp=2 # pressure drop (units: bar)

# Function to create NFMass objects for different components
def create_nfmass_objects(components, C_in, rjr_values, Wrec, Qf):
    return [NFMass(comp, Ci, rjr, Wrec, Qf) for comp, Ci, rjr in zip(components, C_in, rjr_values)]

# Create NFMass objects for different components
nfmass_objects = create_nfmass_objects(components, Ci_in, rjr_values, Wrec, Qf_nf)

Cconc = [nf_mass.Cconci for nf_mass in nfmass_objects]
Cperm = [nf_mass.Cpermi for nf_mass in nfmass_objects]
Qperm = nfmass_objects[0].Qperm  # kg/hr
Qconc= nfmass_objects[0].Qconc # kg/hr

# Calculate Osmotic Pressure
P_osmo_f = OsmoticPressure(c_values, z_values, T).osmotic_pressure_calculation()
P_osmo_p = OsmoticPressure(Cperm, z_values, T).osmotic_pressure_calculation()
P_osmo_c = OsmoticPressure(Cconc, z_values, T).osmotic_pressure_calculation()

d_p=density_calc(T-273, sum(Cperm))

#Calculate Energy consumption 
nf_energy=NfEnergy(P_osmo_c, P_osmo_f, P_osmo_p, dp, d_p, Qperm, Qf_nf, d_in,n)
result=nf_energy.calculate_energy_consumption()
E_el_nf = nf_energy.E_el_nf
for key, value in result.items():
        print(f"{key}: {value}")
       
QHCl_nf=0
Qantsc_nf=0
#%%calculation
"""--------MED--------"""

#Feed concentration
components = ['Na', 'Cl', 'K', 'Mg', 'Ca', 'SO4']
Cin_med = Cperm

#Feed flow rate 
Qf_med =Qperm

#input conditions
T=20
#feed flow density 
d=density_calc(T, sum(Cin_med)) 
Mf_med=Qf_med*d/1000 #Mass flow rate (units: kg/hr)

#assumptions:
T_in=40 #(oC)
T_N=45 #Temperature in the last effect (oC)
N=2 #Number of effects (-)
Cp_w=4182 # specific heat capacity of water (j/kgC)
cp_sol=4184 # specific heat capacity of solution (j/kgC)
T_cw_in=25 #intake cooling water temperature (oC)
T_cw_out=35 #out cooling water temperature (oC)
T_s=70 #steam temperature oC
DT_loss=1 #temperature difference (oC)
dp=0.1  # pressure drop (units: bar)
dp_slurry=1 # pressure drop (units: bar)
npump=0.8 #pump efficiency (units: -)
Cb_out=200 #Brine Concentration leaving effect n (unit: g/l)
Xr=5.5 # brine circulation flow rate (units: -)

#latent heat of motive steam:
if T_s<=55:
    lh_s=2370
elif T_s>55 and T_s<=60:
    lh_s=2358
elif (T_s>60) and (T_s<=65):
    lh_s=2345
elif (T_s>65) and (T_s<=70):
    lh_s=2333
elif (T_s>70) and (T_s<=75):
    lh_s=2321

# Create an instance of the MEDCalculator class
med_dat = MEDCalculator(Qf_med, Mf_med, Cin_med[0], Cin_med[1], Cin_med[2], Cin_med[3], Cin_med[4], Cin_med[5], T_in)

# Call methods to perform calculations
med_dat.salinity_calc()
med_dat.temperature_calc(DT_loss, T_N, T_s)
med_dat.mass_balance_med(Cb_out)
med_dat.performance_parameters(lh_s,  T_cw_in)
med_dat.output_concentration()

# Sum results
#Concentration of brine stream 
Cconc_med = [med_dat.CNa_out, med_dat.CCl_out, med_dat.CK_out, med_dat.CMg_out, med_dat.CCa_out, med_dat.CSO4_out]

#Brine flow rate 
Qout_med=med_dat.Qb

#Distillate water flow rate 
Qprod_med=med_dat.Qdist

#Calculate circulation flow rate 
Qr=Xr*Qf_med

# Calculate density for output concentration
d_b = density_calc(45, sum(Cconc_med))

print("Sum of output concentrations: " + str(round(sum(Cconc_med),2))+"g/l")
print("-----------------------------------------")

# Calculate energy consumption
E_el_med = ((Qf_med * 3.5 + med_dat.QCW * 3600 * 2 + (Qr + med_dat.Qb) * 3.5 + med_dat.Qdist * 1) / (1000 * npump)) * 1e5 / 3600 / 1000  # kWh
print("Electrical energy consumption: " + str(round(E_el_med,2)) + " kWh")

SEC_el = E_el_med / (Qf_med / d)  # kWh/m3 feed
print("Specific energy consumption (electrical) per m3 feed: " + str(round(SEC_el,2)) + " kWh/m3")

SEC_el_prod = E_el_med / (med_dat.Qdist / 1000)  # kWh/m3 dist water
print("Specific energy consumption (electrical) per m3 product (distilled water): " + str(round(SEC_el_prod,2)) + " kWh/m3")
print("-----------------------------------------")

E_th_med = med_dat.Q_Tot
print("Total thermal energy consumption: " + str(round(E_th_med,2)) + " kW")

SEC_th = E_th_med / (Qf_med / d)  # kWh_th/m3
print("Specific energy consumption (thermal) per m3 feed: " + str(round(SEC_th,2)) + " kWh_th/m3")
print("-----------------------------------------")

#Calculate required cooling water 
Qcw_med = med_dat.QCW * 3600 #units: kg/hr
print("Cooling water flow rate: " + str(round(Qcw_med,2)) + " kg/hr")
print("-----------------------------------------")

# Chemical consumption
Cchem_med = 0  # Placeholder for chemical consumption, update as needed

# Print the results
print("Total Chemical Consumption: " + str(round(Cchem_med,2))+"kg/hr")

#%%
#calculation thermal crystallizer 
"""--------TCr--------"""
C_in_mix=[]

#constants
R=8.31446261815324 #gas constant
d_salt=1.974463 #density_salt (NaCl)=1974.5 kg/m3

    # Thermodynamics 
Cp_f=3.14 # Feed specific heat capacity (units: KJ* Kg*oC)
CP_cw=4.187 # Water specific heat capacity (units: KJ* Kg*oC)
UA=45990
LHV_v=2107.92 # kj/kg (gathered from steam table)
LHV_s=2357.69 # kj/kg (gathered from steam table)

#Assumptions
dp=0.1 # pressure drop (units: bar)
dp_slurry=3.5 #pressure drop for slurry streams (units: bar)
dp_f=3.5 #pressure drop for feed (units: bar)
dp_w=1 #pressure drop for qater streams (units: bar)
dp_cw=2 #pressure drop for cooling water (units: bar) 
salt_mois=20 # % moinsture in salt stream (units: %)
npump=0.8 #pump efficiency (units: -)


#Input parameters
T_in=40 #oC
T_top=60 #oC
T_cw_f=25 #oC
T_cw_o=40 #oC
T_op=60 #oC

#Feed flowrate
Qf_tcr=Qconc+ Qout_med#kg/h

#Feed concentration 

Cf_tcr=sum(Cconc_med)*Qout_med/Qf_tcr+sum(Cconc)*Qconc/Qf_tcr
for i in range(len(Cconc)):
    C_in_mix.append(Cconc[i]*Qout_med/Qf_tcr+Cconc[i]*Qconc/Qf_tcr)
    
Cf_tcr_in=C_in_mix #[Na, Cl, K, Mg, Ca, SO4 ]
Cf_s=sum(Cf_tcr_in) #total salt concentration (g/L)
d_sol=density_calc(T_in, Cf_tcr)/1000 #density of feed
Cf_caso4=Cf_tcr_in[4]*MW_CaSO4/MW_Ca #CaSO4 concentration in feed solution (g/L)

#Calculation 
th_cryst_dat=thermal_calc(T_op, Qf_tcr, Cf_tcr, Cf_caso4, T_in, Cf_tcr_in, salt_mois, LHV_v, LHV_s, T_cw_o, T_cw_f)
th_cryst_dat.mass_bal_cryst()
th_cryst_dat.heat_bal_cryst()

#Recovered salt flow rate (kg/h)
M_Nacl=th_cryst_dat.solid_mass
print("Mass flowrate of recovered salt is "+str(round(M_Nacl,2))+"kg/hr")

# Calculation of the concentration of different ions in the solution
th_cryst_dat_2=conc_cal(Qf_tcr, M_Nacl , 'Na',Cf_tcr_in[0], 'cl',Cf_tcr_in[1],'k', Cf_tcr_in[2], 'mg', Cf_tcr_in[3], 'ca', Cf_tcr_in[4], 'so4', Cf_tcr_in[5], T_in)
th_cryst_dat_2.molarity()
th_cryst_dat_2.conc_salt_comp()
th_cryst_dat_2.salt_conc()

#Evaporation mass flow rate (water recovery), (kg/h)
Q_evap_mass_tcr=th_cryst_dat.ev_mass
print("Mass flowrate of recovered water is "+str(round(Q_evap_mass_tcr,2))+"kg/hr")
print("-----------------------------------------")
#Salt stream concentration (g/L)
Csalt_out=[th_cryst_dat_2.CNa,th_cryst_dat_2.CCl, th_cryst_dat_2.CK, th_cryst_dat_2.CMg, th_cryst_dat_2.CCa, th_cryst_dat_2.CSO4]

#Cooling water requirements (kg/h)
Qcw_tcr=th_cryst_dat.cw_mass
print("Mass flowrate of required cooling water is "+str(round(Qcw_tcr,2))+"kg/hr")
print("-----------------------------------------")
#Calculate energy consumption 
heat_req=th_cryst_dat.heat_req
E_el_th_Cr, E_th_th_Cr, SEC_el_f, SEC_el_NaCl, SEC_el_w = calculate_energy(Qf_tcr, Q_evap_mass_tcr, Qcw_tcr, M_Nacl, heat_req, d_sol, dp_f, dp_w, dp_slurry, dp_cw, npump)
print(f"SEC_el_prod is {SEC_el_w} KWh/m3 product")
print(f"SEC_el_prod2 is {SEC_el_NaCl} KWh/kg of NaCl product")
print("SEC_th_prod is "+str( round((E_th_th_Cr/(Qf_tcr/1000)),2))+" KWh/m3 intake")

# Chemical consumption
Cchem_tcr = 0  # Placeholder for chemical consumption, update as needed


#%%Indicators:
class indic:
    
    def __init__(self, tech, Qout, Qin, Qprod_1, prod_1_name, Qprod_2, prod_2_name, E_el, E_th, Cin, Cout, chem1,chem2):
        self.tech=tech
        self.Qout=Qout
        self.Qin=Qin
        self.E_el=E_el
        self.E_th=E_th
        self.Cin=Cin
        self.Cout=Cout
        self.Qprod_1=Qprod_1
        self.Qprod_2=Qprod_2
        self.prod_1_name=prod_1_name
        self.prod_2_name=prod_2_name
        self.chem=chem1+chem2
        
        
    def techn_indi(self):        
        if self.prod_1_name=="water": 
            self.Water_rr=self.Qprod_1/self.Qin*100 # %
            self.Qwater=self.Qprod_1
        elif self.prod_2_name=="water":
            self.Water_rr=self.Qprod_2/self.Qin*100 # %
            self.Qwater=self.Qprod_2
        else:
            self.Qwater=0
            self.Water_rr=0
            print("no water production by " + self.tech)
        
#%% Sumsummarize results   

tec1=indic("NF", Qconc, Qf_nf, Qperm, "none", 0, "none", E_el_nf, 0, Ci_in, Cconc, QHCl_nf, Qantsc_nf)   
tec1.techn_indi()

tec2=indic("MED", Qout_med, Qf_med, Qprod_med, "water", 0, "none",  E_el_med, E_th_med, Cin_med, Cconc_med, Cchem_med,0)
tec2.techn_indi()

tec3=indic("Thermal Cryst", 0, Qf_tcr, Q_evap_mass_tcr,"water", M_Nacl, "NaCl", E_el_th_Cr, E_th_th_Cr, Cf_tcr, 0,0,0 )
tec3.techn_indi()


##list results 
tec_names=[tec1.tech, tec2.tech, tec3.tech]
E_el_all=[tec1.E_el, tec2.E_el, tec3.E_el]
E_th_all=[tec1.E_th, tec2.E_th, tec3.E_th]
Cout_all=[tec1.Cout, tec2.Cout, tec3.Cout]
Qout_all=[tec1.Qout, tec2.Qout, tec3.Qout]
Qchem_all=[tec1.chem,tec2.chem, tec3.chem]

sum_el_en=sum(E_el_all)
sum_th_en=sum(E_th_all)
#%%
Qsw=Qsw
#Technical indicators 
#T-1) quantity of water production 
Qw_tot=tec1.Qwater+tec2.Qwater+ tec3.Qwater
#print("total water production is " + str(Qw_tot))
abs_Qw=Qw_tot
#T-1b) Water recovery  
rec=Qw_tot/Qsw*100 #%
#T-2) Specific electrical energy consumption #kwh/kg of desalinated water 
SEC=sum(E_el_all)/Qsw
print("specific electrical consumption is " +str(SEC)+ " Kwh/kg of desalinated water")

#T-3)Brine production -> kg of brine/ kg of seawater 
Qbrine_f=0
for i in range(len(tec_names)):
    if tec_names[i]=="Thermal Cryst":
        if Cout_all[i]>(constants.Csw):
            Qbrine_f=Qbrine_f+Qout_all[i]
        else:
            Qbrine_f=Qbrine_f+0
Q_br_prod=Qbrine_f/Qsw
print("brine production is "+str(Q_br_prod)+" kg of brine/kg of seawater")
#------------------------------------------------------------------------------
#%%
#Environmental 
#Env-1)Climate change impact ->Carbon dioxide emission kg co2
emis=(sum(E_el_all)*constants.CO2_el)
emis_t=((sum(E_el_all)*constants.CO2_el)+(sum(E_th_all)*constants.CO2_st))*constants.hr
print("Total specific Carbon dioxide emission "+str(emis_t)+" kg co2")
#Env-2) Resource utilization -> chemical consumption in kg of chemical/kg of seawater 
Chem_cons=(sum(Qchem_all))/Qsw #check units and convert ml/l to kg with densities 
print("Total chemical consumption is "+str(Chem_cons)+" kg of chemical/kg of seawater ")

#%%
#Economic 
#Input data 
hr=24*300 #hours/year
lf= 20 #years
r=0.06 # interest rate
inf=0.02 # inflation rate 

#prices 
el_pr=0.253 #euro/kwh
s_pr=0 #euro/kwh
co2_em_c=23 #euro/ton co2 
nacl_pr=66/1000 # euro/kg 
hcl_pr=1040.0/180 #euro/l 1M solution 
w_pr=0.001 #euro/kg
na2so4_pr=140*0.83/1000 #euro/kg
mgoh2_pr=1000/1000 #euro/kg
caoh2_pr=125/1000#euro/kg
naoh_pr=6840/950#euro/L 1M NaOH solution  
antisc_pr=0.002 #euro/ml
cw_pr=0.118/1000#euro/kg
m_salt_pr=5/1000#euro/kg

#density
d_naoh=1.04 #kg/l for 1M solution 
d_hcl=1.0585#kg/l for 1M solution 

#Assumptions for CAPEX 
inst_percent=0.25
buildings_percent=0.2
land_percent=0.06
indirect_c_percent=0.15
workinf_c_percent=0.2
capex_assumptions=[inst_percent,buildings_percent, land_percent, indirect_c_percent, workinf_c_percent]


#Assumptions for OPEX
main_c_percent=0.03  # % of the fixed-capital investment
oper_sup_c_percent=0.05  # % of maintenance 
oper_lab_c_percent=0.15    # % of annual OPEX
super_c_percent=0.15 # % of operating labor 
lab_c_percent=0.15 # % of operating labor 
pat_c_percent=0.03 # % of annual OPEX
fix_char_percent=0.05 # % of annual OPEX
over_c_percent=0.05 # %of annual OPEX
norm_factor =(1 -oper_lab_c_percent-oper_lab_c_percent*super_c_percent- oper_lab_c_percent*lab_c_percent-pat_c_percent-fix_char_percent-over_c_percent)
economic_assumptions=[main_c_percent,oper_sup_c_percent, oper_lab_c_percent, super_c_percent, lab_c_percent, pat_c_percent, 
                      fix_char_percent, over_c_percent, norm_factor ]

# Initialize values
CAPEX=0
OPEX=0
CO2_c=0

 # Create list with capital cost for each unit.
capex_list=[]
 # Create list with operating for each unit.
opex_list=[]

#summarise results for utilities 
el_conc=[E_el_nf,E_el_med,E_el_th_Cr] 
s_conc=[0,E_th_med,E_th_th_Cr]
chem1_conc=[Qantsc_nf,Cchem_med,Cchem_tcr]
chem1_pr=[0.002,0,0]
chem2_conc=[0,0,0]
chem2_pr=[0,0,0]
cw_conc=[0,Qcw_med, Qcw_tcr]
wat_conc=[0,0, 0]

if sum(wat_conc)==0:
    Q_w_in=0

# Calculation of scale-up equipment cost 
# Import equipment cost for reference scenario from constants function
eq_c=[constants.eq_c_nf, constants.eq_c_med, constants.eq_c_th_cr]   
    # Capacity of reference scenario
Mf_basic_sc=[constants.eq_c_nf,constants.eq_c_med,constants.eq_c_th_cr]
Mf_sce=[Qsw, Qperm,Qf_tcr]
    
    # Calculation of the new equipment cost
for i in range(len(eq_c)):
    if Mf_basic_sc[i]!=Mf_sce[i]:
        eq_c[i]=scaleup.scaleup_eq(eq_c[i],Mf_basic_sc[i],Mf_sce[i],tec_names[i])
        
#Cost calculation 
for i in range(len(eq_c)):
    total_econom=econom(eq_c[i], el_conc[i], s_conc[i], chem1_conc[i], chem1_pr[i],chem2_conc[i], chem2_pr[i], cw_conc[i], wat_conc[i])
    total_econom.capex_calc(capex_assumptions)
    total_econom.opex_calc(hr, el_pr, s_pr, cw_pr, w_pr, economic_assumptions)
    CAPEX=total_econom.t_capital_inv
    OPEX=total_econom.opex
    opex_list.append(total_econom.opex)
    capex_list.append(total_econom.t_capital_inv)    
print("Total operating cost (OPEX) is "+str(OPEX)+ " Euro/year") 
print("Total investment cost (CAPEX) of system is " + str(round(CAPEX))+ " Euro")    
print("-----------------------------------------")

#amortisation factor 
a=(constants.r*(1+constants.r)**constants.lf)/((1+constants.r)**constants.lf-1)

"""Revenue calculation"""
#Input data 
reve_t=0
reve_list=[]
prd=[10,2,3,0,0,0]    
prd_name= ["Water", "NaCl", "Mg(OH)2", "Na2SO4", "NaOH", "HCl"]   

#Revenue calculation
for i in range(len(prd)):
    rev_calc=revenue(prd[i], prd_name[i])    
    rev_calc.rev(hr, w_pr, nacl_pr, mgoh2_pr,na2so4_pr, naoh_pr, hcl_pr)
    print("Revenues from selling product " + prd_name[i]+" are " + str(round(rev_calc.rev_prd,2))+" Euro/year")
    reve_t = reve_t+rev_calc.rev_prd
    reve_list.append(rev_calc.rev_prd)

#%%present results
#Summary tables 
    #Table C: Ion concentration 
sum_table_C={'F1: Seawater': Ci_in,
           'F2: NF permeate': Cperm, 
           'F3: NF concentrate': Cconc,
           'F4: MED brine': Cconc_med, 
           'F5: MED water': [0,0,0,0,0,0],
           'F6: Thermal Cryst water': [0,0,0,0,0,0],
           'F7: Thermal Cryst salt': Csalt_out,
           }
sum_table_C=pd.DataFrame(sum_table_C)
print(sum_table_C)

    #Table D: Density
sum_table_d={'F1: Seawater': density_calc(25,sum(Ci_in))/1000,
           'F2: NF permeate': density_calc(25,sum(Cperm))/1000, 
           'F3: NF concentrate': density_calc(25,sum(Cconc))/1000,
           'F4: MED brine': density_calc(25,sum(Cconc_med))/1000, 
           'F5: MED water': 1,
           'F6: Thermal Cryst water': 1,
           'F7: Thermal Cryst salt': density_calc(25,sum(Csalt_out))/1000,
           }
sum_table_d=pd.DataFrame(sum_table_d, index=['density'])
print(sum_table_d)

        #Table F: Flow rates  
sum_table_f={'F1: Seawater': round(Qsw,2),
           'F2: NF permeate': round(Qperm,2), 
           'F3: NF concentrate': round(Qconc,2),
           'F4: MED brine': round(Qout_med,2), 
           'F5: MED water': round(Qprod_med,2),
           'F6: Thermal Cryst water': round(Q_evap_mass_tcr,2),
           'F7: Thermal Cryst salt':round( M_Nacl,2),
           'F8: NF antiscalant': round(Qantsc_nf,2), 
           'F9: MED Cooling water': round(Qcw_med,2), 
           'F10: Thermal Cryst Cooling water': round(Qcw_tcr,2), 
           }
sum_table_f=pd.DataFrame(sum_table_f, index=['flow rate'])
print(sum_table_f)

#sankey diagram 
import plotly.graph_objects as go
from plotly.offline import plot

#Figure 1: Sankey diagram for Example: Mass flow rates
fig = go.Figure(data=[go.Sankey(
    node = dict(
      pad = 15,
      thickness = 20,
      line = dict(color = "black", width = 0.5),
      label = ["Seawater", "Water", "NaCL",  "NF", "MED", "Cryst", "Mixed Salt"],
      color = "blue"
    ),
    link = dict(
      source = [0, 3,3,4,4, 5,5], 
      target = [3,4,5,5,1,1,6],
      value = [Qsw, Qperm, Qconc, Qout_med, Qprod_med, Q_evap_mass_tcr, M_Nacl]
  ))])
color_for_nodes = ["lightsteelblue","darkcyan","maroon", "midnightblue", "midnightblue", "midnightblue", "maroon"]
fig.update_traces(node_color = color_for_nodes)
fig.update_layout(title_text="Sankey diagram for Example: Mass flow rates ", font_size=15)
fig.show()
plot(fig)

#%% Results to excel 
dfel=pd.DataFrame(E_el_all ,tec_names)
dfeth=pd.DataFrame(E_th_all, tec_names)
dfind=(Qw_tot, abs_Qw, rec, sum_el_en, sum_th_en, emis_t, Q_w_in,  OPEX, CAPEX)
dfprod=(Qw_tot,M_Nacl, 0, 0, 0, 0, 0)

dfprodn=("Water (kg/hr)", "NaCl (kg/hr)", "Mg(OH)\u2082 (kg/hr)", "Ca(OH)\u2082 (kg/hr)", "Na\u2082SO\u2084(kg/hr)", "1M NaOH (kg/hr)","1M HCl(kg/hr)")
ind=np.array(["Water production", "absolute water production", "water recovery","Total electrical consumption", "Total thermal energy consumption", "Carbon dioxide emission kg co2/year",
              "Water footprint","OPEX", "CAPEX"])
units=pd.DataFrame(["kg/hr", "kg/hr", "%","KWh", "KWh"," kg CO\u2082/year","kg/hr","€/year", "€"])
dfind_t=pd.DataFrame(data=dfind, index=ind)
dfprodn=pd.DataFrame(data=dfprod, index=dfprodn)
with pd.ExcelWriter('results_example .xlsx') as writer:
    sum_table_f.to_excel(writer,sheet_name="example ")
    sum_table_C.to_excel(writer,sheet_name="example ", startcol=0, startrow=4, header=True)
    dfel.to_excel(writer,sheet_name="example ", startcol=0, startrow=14, header=True)
    dfeth.to_excel(writer,sheet_name="example ", startcol=2, startrow=14, header=True)
    dfprodn.to_excel(writer,sheet_name="example ", startcol=0, startrow=23, header=True)
    dfind_t.to_excel(writer,sheet_name="indicators")
    units.to_excel(writer,sheet_name="indicators",startcol=2, startrow=0, header=True)


#Import functions 
from desalsim.density_calc import density_calc
from desalsim import constants
from desalsim import scaleup

from desalsim.nanofiltration_unit_f import NFMass
from desalsim.nanofiltration_unit_f import OsmoticPressure
from desalsim.nanofiltration_unit_f import NfEnergy

from desalsim.mfpfr_unit_f import MFPFRCALC
from desalsim.mfpfr_unit_f import HClAddition
from desalsim.mfpfr_unit_f import energycons

from desalsim.ed_unit_f import ElectrodialysisCalc

from desalsim.edbm_unit_f import EDBMCalc

from desalsim.economic_f import econom
from desalsim.economic_f import revenue

import numpy as np
import pandas as pd
import math


#%%Constants

#PRICES
ele_price=0.18 # euro/kwh
steam_price=0.01 # euro/kwh
HCl_price=0.000632058 #euro/l
NaOH_price=0.33 #euro/l
water_price=0.001 # euro/kg
Na2SO4_price=0.05 #euro/kg
NaCl_price=0.066 #euro/kg
MGOH_price=1 #euro/kg
CaOH_price=0.125 #euro/kg


#Molecular weight 
    #molecular weight
MW_Na=constants.MW_Na
MW_Cl=constants.MW_cl
MW_SO4=constants.MW_so4
MW_K=constants.MW_K
MW_Ca=constants.MW_Ca
MW_Mg=constants.MW_Mg
MW_HCO3=constants.MW_HCO3
MW_H=1.00784
MW_OH=17.008
MW_H2O=MW_H+MW_OH
MW_values = [MW_Na, MW_Cl, MW_K, MW_Mg, MW_Ca, MW_SO4]


MW_MgOH=58.3197
MW_CaOH=74.09
MW_Na2SO4=142.039458
MW_HCl=36.458
MW_NaOH=39.997
MW_NaCl=58.442729
MW_Na2SO4=142.04  
MW_Na2SO4_10h20=322.192258                                                                                                   #Molecular weight of sodium sulfate : g/mol
MW_KCl=74.5513

#constants 
d_Na2SO4=2.66*1000                                                                                                  #Density of sodium sulfate (25oC) : kg/m^3 (source:???) 
Cp_Na2SO4=128.2/MW_Na2SO4*1000   
d_NaCl=2.16*1000                                                                                                    #Density of NaCl (25oC) : kg/m^3 (source:https://pubchem.ncbi.nlm.nih.gov/compound/sodium_chloride#section=Solubility) 
Cp_NaCl=(50.5*MW_NaCl)*1000    
d_w=1*1000                                                                                                      #Density of water (25oC) : kg/m^3 (source:Wikipedia) 
Cp_w=4.1813*1000   
d_KCl=1.98*1000

#input data
#emissions
CO2_el=0.275 # kg/kwh
CO2_st=0#.255 # kg/kwh

#input data
T=20+273 #Operating temperature (units: K)
#Feed concentration
components = ['Na', 'Cl', 'K', 'Mg', 'Ca', 'SO4']
Ci_in = [12.33, 21.67, 0.45, 1.39, 0.45, 3.28]
z_values = [1, -1, 1, 2, 2, -2]
mg_in = sum(Ci_in )
#Feed flow density 
d_in = density_calc(T-273, mg_in)  # kg/m3

#Feed flowrate
Qsw = 3000 / 24 * d_in #m3/d


#%%
#Calculations
"""--------Nanofiltration--------"""

Qf = Qsw  # kg/hr
#Constants
R=8.314 #gas constant (units: J / mol·K)

#Asuumptions  
rjr_values = [0.16, 0.29, 0.21, 0.98, 0.95, 0.98] #Ions rejection rates based on membrane characteristics (units: -)
Wrec = 0.7 # Water recovery based on membrane characteristics (units: -)
n=0.8 #pump efficiency (units: -)
dp=2 # pressure drop (units: bar)

# Function to create NFMass objects for different components
def create_nfmass_objects(components, C_in, rjr_values, Wrec, Qf):
    return [NFMass(comp, Ci, rjr, Wrec, Qf) for comp, Ci, rjr in zip(components, C_in, rjr_values)]

# Create NFMass objects for different components
nfmass_objects = create_nfmass_objects(components, Ci_in, rjr_values, Wrec, Qf)

# Components concentrattion in concentrate stream 
Cconc = [nf_mass.Cconci for nf_mass in nfmass_objects]
# Components concentrattion in permeate stream 
Cperm = [nf_mass.Cpermi for nf_mass in nfmass_objects]
# Permeate stream mass flow rate
Qperm = nfmass_objects[0].Qperm  # kg/hr
# Concentrate stream mass flow rate
Qconc = nfmass_objects[0].Qconc  # kg/hr
print("Permeate stream flow rate is "+str(round(Qperm,2))+"kg/hr")
print("Permeate stream total concentration is "+str(round(sum(Cperm),2))+"g/l")
print("-----------------------------------------")
print("Concentrate stream flow rate is "+str(round(Qconc,2))+"kg/hr")
print("Concentrate stream total concentration is "+str(round(sum(Cconc),2))+"g/l")
print("-----------------------------------------")

# Calculate Osmotic Pressure
P_osmo_f = OsmoticPressure(Ci_in, z_values, T).osmotic_pressure_calculation()
P_osmo_p = OsmoticPressure(Cperm, z_values, T).osmotic_pressure_calculation()
P_osmo_c = OsmoticPressure(Cconc, z_values, T).osmotic_pressure_calculation()

d_p=density_calc(T-273, sum(Cperm))

#Calculate Energy consumption 
nf_energy=NfEnergy(P_osmo_c, P_osmo_f, P_osmo_p, dp, d_p, Qperm, Qf, d_in,n)
result=nf_energy.calculate_energy_consumption()
E_el_nf = nf_energy.E_el_nf
for key, value in result.items():
        print(f"{key}: {value}")
        
QHCl_nf=0
Qantsc_nf=0
    
#%%
"""--------MF-PFR--------"""

#constants
kps_MgOH=5.61*0.000000000001 #product solublity of Mg(OH)2
kps_CaOH=5.5*0.000001 #product solublity of Ca(OH)2
d_mgoh_2=2.34 #Mg(OH)2 density (units: kg/l)
d_caoh_2=2.211 #Ca(OH)2 density (units: kg/l)

#assumptions
dp=0.5 # pressure drop (units: bar)
dp_HCl=0.3# pressure drop HCl solution (units: bar)
npump=0.8 #pump efficiency (units: -)

# Define the input parameters
T=20+273.15 #Temperature (units: K)
C_NaOH = [1, 1]  # Concentration of NaOH solution for step 1 and step 2 in MOL/L
conv = [95, 93]  # Conversion rate for step 1 and step 2
Cin_mfpfr = Cconc  # Concentrations of [Na, Cl, K, Mg, Ca, SO4]
Qin_mfpfr = Qconc # Flow rate in l/hr

# Calculate the density of the input
d_in = density_calc(25, sum(Cin_mfpfr)) / 1000

# Create an instance of the inputpar class with the defined parameters
mfpfr_dat = MFPFRCALC(Qin_mfpfr, Cin_mfpfr, *C_NaOH, *conv)

# Call the calc_step1 and calc_step2 methods to calculate the necessary values
mfpfr_dat.calc_step1(kps_MgOH, d_mgoh_2)
mfpfr_dat.calc_step2(d_mgoh_2, d_caoh_2 )
ph_2=mfpfr_dat.ph_2

# Calculate the total outlet concentration and the concentration of sulfate ions
Cour_mfpfr = sum([mfpfr_dat.CNa_out_2, mfpfr_dat.CCa_out_2, mfpfr_dat.CCl_out_2, mfpfr_dat.CK_out_2, mfpfr_dat.CMg_out_2, mfpfr_dat.CSO4_out_2]) # mol/l
CSO4_out_2 = mfpfr_dat.CSO4_out_2 # mol/l

# Calculate the concentration of sodium chloride ions
CNa_out_2 = mfpfr_dat.CNa_out_2
Cnacl_out = CNa_out_2 - 2 * CSO4_out_2

# Get the molar masses of the compounds
M_MgOH2_1 = mfpfr_dat.M_MgOH2_1
M_CaOH2 = mfpfr_dat.M_CaOH2_2
M_MgOH2 = mfpfr_dat.M_MgOH2_2

# Create a list of the outlet concentrations in mol/l
Cout_all_m = [mfpfr_dat.CNa_out_2, mfpfr_dat.CCl_out_2, mfpfr_dat.CK_out_2, mfpfr_dat.CMg_out_2, mfpfr_dat.CCa_out_2, mfpfr_dat.CSO4_out_2]

# Calculate the outlet concentrations in g/l
MW = [MW_Na, MW_Ca, MW_Cl, MW_K, MW_Mg, MW_SO4]
Cout_mfpfr_g = [Cout_all_m[i] * MW[i] for i in range(len(Cout_all_m))] # g/l

#Outlet flow rate 
Qout_2 = mfpfr_dat.Qout_2

# Calculate the chemical consumption of NaOH
QNAOH = mfpfr_dat.QNaOH_1 + mfpfr_dat.QNaOH_2_add + mfpfr_dat.QNaOH_2_st # convert to kg

#hcl to decrease ph to 7
HCl_conc=1 #l of HCl 1M
unit = HClAddition(Qout_2, Cout_all_m, MW_Cl, ph_2, HCl_conc)

# Call the calculate_HCl_addition method
QHCl, Cout_mfpfr_g = unit.calculate_HCl_addition(Cout_mfpfr_g)

# Print the volume of HCl added and the outlet concentration of chloride ions
print(f"HCl flow rate is {round(QHCl,2)} l/hr")
print(f"NaOH flow rate is {round(QNAOH,2)} l/hr")
print("-----------------------------------------")

#Calculate final outlet flow rate
Qout_f=Qout_2+QHCl #l/h
d_out_s=density_calc(25,round(sum(Cout_mfpfr_g),2))/1000 #kg/m3
M_mfpfr_out=Qout_f*d_out_s #kg/h

print("Mg(OH)2 mass flow rate is "+str(round(M_MgOH2_1,2))+"kg/hr")
print("Ca(OH)2 mass flow rate is "+str(round(M_CaOH2,2))+"kg/hr")
print("Total effluent flow rate is "+str(round(M_mfpfr_out,2))+"kg/hr")
print("Total effluent flow rate is "+str(round(Qout_f,2))+"kg/hr")
print("-----------------------------------------")

#electicity consumption
    # Create an instance of the inputpar class with the defined parameters
Epump_1, Epump_2=energycons.energycalc(mfpfr_dat.Qout_2, QNAOH, Qin_mfpfr, mfpfr_dat.QNaOH_1, mfpfr_dat.QNaOH_2_add, mfpfr_dat.QNaOH_2_st, dp, npump)

    #Electricity consumption for pumping , KWh
E_el_mfpf=(Epump_1+Epump_2+(QHCl*dp_HCl)*1e5/3600/(1000*npump))/1000
print("Total electricity energy consumption is "+str(round(E_el_mfpf,2))+ " KW")

    #Electricity consumption for filtration unit 
E_fil=scaleup.scaleup(0.5, 0.3*1000, M_mfpfr_out)

    #Total electricity consumption, KWh
E_el_mfpf=E_el_mfpf+E_fil

    #Specific energy consumption per kg of Mg(OH)2, KWh/kg of Mg(OH)2
SEC_el_prod=(E_el_mfpf)/(M_MgOH2)
print("Specific energy consumption per product is "+str(round(SEC_el_prod,2))+" KWh/kg product ")

    #Specific energy consumption per feed, KWh/m3 of feed
SEC_el_feed=(E_el_mfpf)/(Qin_mfpfr/1000)
print("Specific energy consumption per brine intake is "+str(round(SEC_el_feed,2))+" KWh/m3 of feed ")

#%%
"""--------Electrodialysis--------"""
MWs = 58.44  # Molecular weight of NaCl (g/mol)
MWw=18.01528# Molecular weight of water (g/mol)

#Assumptions 
R = 0.002  # Resistance of rinse stream (ohm)
Rp = 0.015  # Resistance of polarization (ohm)
A = 1.1  # Active area of cell-pair (m^2)
F = 96485.3329  # Faraday constant (C/mol)
T=20+273.15 #Inlet/operating temperature (K)
dp=1 #pressure drop (units: bar)
npump=0.8 #pump efficiency (units: -)
rho_w=1000 # water desnity kg/m3

#model parameters 
D=1.61e-9 #Diffusion coefficient (m^2/s)
tcu=0.5
veloc=8.9e-7 #m2/s
h=0.5 #mm
Sh=18

#Input data 
Sc_i = sum(Cperm)  # Salinity at concentrate inlet (g/kg)
Sc_o = 200  # Salinity at concentrate outlet (g/kg)
Sd_o=20 # Salinity at doluate outlet (g/kg)
Sd_i=Sc_i # Salinity at diluate inlet (g/kg)
N = 50 # Number of computational cells per cell-pair
Vcp = 8  # Applied voltage (V)
Vel=2.1 #Voltage across the electrodes 
Ij=400 #Am2
Mem_eff=0.64 #Membrane efficiency
Ncp=1 #Number of identical parallel cell-pairs 

#Feed flow rate L/h
d_in_ed=(density_calc(T-273, Sc_i)/1000)
Qed_in=Qperm
Qed_in_c=Qed_in/17/Ncp
Qed_in_d=Qed_in*16/17/Ncp

# Effective cell-pair area (m^2)
Acp =scaleup.scaleup(24, 1000, Qed_in) 
Acp_tot=Acp

# Initializations
Sc = np.zeros(N)
Sd = np.zeros(N)
Ns_c = np.zeros(N)
Ns_d = np.zeros(N)
Nw_c = np.zeros(N)
Nw_d = np.zeros(N)
Js = np.zeros(N)
Jw = np.zeros(N)
Mw_in_d_l=np.zeros(N)
Ms_d=np.zeros(N)
Mw_d=np.zeros(N)
M_d=np.zeros(N)
M_c=np.zeros(N)
Q_c=np.zeros(N)
Q_d=np.zeros(N)

# Set initial values salt stream 
Sc[0] = Sc_i
Ns_c[0]=Qed_in_c*d_in_ed*Sc[0]/MWs #mol/hr
Ms_in_c=Qed_in_c*d_in_ed*Sc_i/1000 #kg salr/hr
Mw_in_c=Qed_in_c*d_in_ed-Ms_in_c #kg water/hr
Nw_c[0]=Mw_in_c*1000/MWw #mol/hr
M_c[0]=Mw_in_c+Ms_in_c
Q_c[0]=Qed_in_c


# Set initial values diluate stream 
Csw= [17.17, 25.47, 0.57, 0.04, 0.03, 0.10]
Sd[0] = Sc_i #g/kg 
Ns_d[0]=Qed_in_d*d_in_ed*Sd[0]/(MWs) #mol/s
Ms_in_d=Qed_in_d*d_in_ed*Sd[0]/1000 #kg salr/hr
Mw_in_d=Qed_in_d*d_in_ed-Ms_in_d #kg water/hr
Nw_d[0]=Mw_in_d*1000/MWw #mol/hr
Mw_in_d_l[0]=Mw_in_d
Ms_d[0]=Ms_in_d
Mw_d[0]=Mw_in_d
M_d[0]=Mw_in_d+Ms_in_d
Q_d[0]=Qed_in_d

# Initialize the Acp_tot array
Acp_tot_j=Acp_tot/N
ed_em = ElectrodialysisCalc()

# Iterate over cells
for j in range(1, N):
    #Calculate salinity change 
    concentration_diff = Sc[j - 1] - Sd[j - 1]
    Sc[j] = Sc[j - 1] + (Sc_o - Sc_i) / (N - 1)
    Sd[j] = Sd[j - 1] + (Sd_o - Sd_i) / (N - 1)
    
    #Calculate net salt flux 
    Js[j] = (ElectrodialysisCalc.Ts_cp(Sd[j - 1]) * Ij / F - 
             (ElectrodialysisCalc.Ls_cp(Sc[j - 1], Sd[j - 1])) * concentration_diff)

    #calculate net water flux 
    Jw[j] = (ElectrodialysisCalc.Tw_cp(Sc[j - 1], Sd[j - 1]) * Ij / F +
             ElectrodialysisCalc.Lw_cp(Sc[j - 1]) * (ElectrodialysisCalc.p_osmo(Sc[j - 1], T, MWs) - 
                                                      ElectrodialysisCalc.p_osmo(Sd[j - 1], T, MWs)))
    
    #Calculate he total concentrate and dilute molar flow rates
    Ns_c[j] = Ns_c[j - 1] + Acp_tot_j * Js[j]
    Ns_d[j] = Ns_d[j - 1] - Acp_tot_j * Js[j]

    Nw_c[j] = Nw_c[j - 1] + Acp_tot_j * Jw[j]
    Nw_d[j] = Nw_d[j - 1] - Acp_tot_j * Jw[j]

    # Update the flow rates of the concentrate and dilute streams
    Q_c[j] = Nw_c[j] * MWw / (rho_w * (1 - Sc[j] / 1000))
    Q_d[j] = Nw_d[j] * MWw / (rho_w * (1 - Sd[j] / 1000))


Cc_na_f=Sc[N-1]/MWs*constants.MW_Na
Cc_cl_f=Sc[N-1]/MWs*constants.MW_cl
Sc_out=[Cc_na_f, Cc_cl_f]

#Calculate the concnetrate stream flow rate 
Mc=(Ns_c[N-1]*MWs/1000+Nw_c[N-1]*MWw/1000) #(kg/hr)
print("Mass flowrate concentrate stream is "+str(round(Mc,2))+ " kg/hr")

dc_out=density_calc(T-273, Sc[N-1])/1000 #(kg/l)
Qc=Mc/dc_out #concnetrate stream volume flow rate (l/hr)
i=2

for i in range(2,len(Csw)):
    Sc_out.append(Csw[i]*Qed_in_c/Qc)

print("Volume flowrate concentrate stream is "+str(round(Qc,2))+" l/hr")
print("The total effluent concentration concentrate stream  is " + str(round(Sc[N-1],2))+"g/kg")
print("-----------------------------------------")

#Calculations for diluate stream 
Md=(Ns_d[N-1]*MWs/1000+Nw_d[N-1]*MWw/1000) #mass flow rate (kg/hr)
print("Mass flowrate of diluate stream is "+str(round(Md,2))+" kg/hr")

Sd_f=Sd[N-1]
Cd_na_f=Sd_f/MWs*constants.MW_Na
Cd_cl_f=Sd_f/MWs*constants.MW_cl
dd_out=density_calc(T-273, Sd[N-1])/1000 #density of diluate stream
Qd=Md/dd_out #diluate stream volume flow rate (l/hr)

print("volume flowrate diluate stream is "+str(round(Qd,2))+" l/hr")
print("The total effluent concentration dilute is " + str(round(Sd[N-1],2))+"g/kg")

Sd_out=[Cd_na_f, Cd_cl_f]

for i in range(2,len(Csw)):
    Sd_out.append(Csw[i]*Qed_in/Qd)
print("-----------------------------------------")

#Energy consumption 
#power required 
Ws = 0
for j in range(N):
     Ws += Ij * Acp_tot_j * (Ncp * Vcp + Vel)
print("Power required is "+str(round(Ws/1000,2))+"KW")

#Calculate energy consumption for pumping 
Ppump_ed=(Qed_in_d*1+Qed_in_c*1+Qc*2+Qd*1)/1000/3600*1e5/npump
Eel_t_ed=Ws/1000+Ppump_ed/1000
print("Total energy consumption is "+str(round(Eel_t_ed,2))+"KW")
sec_ed=Eel_t_ed/(Qed_in/1000)
print("Specific energy consumption of Electrodialysis (ED) is "+str(round(sec_ed,2))+"KW/m3 feed")

#%%
"""--------Electrodialysis with Bipolar Membranes--------"""
#constants
I_d=400 # The electricl current desnity Am2

#A=0.16 #active area of the membrane across which ion permeation occurs (A)

F=96485.3 #Coulombs/mol
R_const=8.314462618 #kg⋅m2⋅s−2⋅K−1⋅mol−1
# R_int=0.28 #ohm cm2
R_int=45 #ohm cm2
z=1
npump=0.8 #pump efficiency (units: -)
dp_f=0.1 #pressure drop for feed (units: bar)
dp_r=1.2 #pressure drop for recycling (units: bar)
dp_p=1 #pressure drop for extracting products (units: bar)
T_in=25

#Membrane characteristics

Cm_bp_H= 0.0000001 #mol/l 
Cm_bp_OH= 0.0000001 #mol/l 


#input data
r=1-0.4/(5-0.4) #recycling rate 
rs=1-1/(5-1) #recycling rate 
ph_s=4.71 #pH salt channel (units: -)
ph_b=7#pH base channel (units: -)
ph_a=7#pH acid channel (units: -)
d_a=1 #density acid channel (units: kg/l)
d_s=1 #density salt channel (units: kg/l)
d_b=1 #density base channel (units: kg/l)

#Feed flow rate L/h
Q_in_edbm=M_mfpfr_out+Mc

#Feed concentration g/L
Cin_edbm=sum(Cout_mfpfr_g)*M_mfpfr_out/Q_in_edbm+Sc_o*Mc/Q_in_edbm
C_in_mix=[]
for i in range(len(Cconc)):
    C_in_mix.append(Cout_mfpfr_g[i]*M_mfpfr_out/Q_in_edbm+Sc_out[i]*Mc/Q_in_edbm)
    
Cin_edbm=C_in_mix #[Na, Cl, K, Mg, Ca, SO4 ]
Cin_edbm.extend([0, 10**(-ph_s), 3.01551E-11])
d_in=density_calc(T_in,sum(Cin_edbm))/1000

C_b_in=[0,0,0,0,0,0,0,10**(-ph_b), 10**(-(14-ph_b))]
C_a_in=[0,0,0,0,0,0,0,10**(-ph_a), 10**(-(14-ph_a))]
#Calculate water quantity in inflow 
Mw_in=Q_in_edbm/d_in 

#Set number of triplets 
N_trip=50*47

#Set membrane area based on the feed flow rate 
A=0.4

#Initialize concentration of Na in salt channel 
Cna_s=[]

#Create an instance of the EDBMCalc class with the defined parameters
edbm_dat=EDBMCalc(Q_in_edbm, A, I_d, N_trip, Cin_edbm, C_b_in, C_a_in, T_in )

# Call the necessary methods to calculate values
edbm_dat.flowrate()
edbm_dat.in_mass_flow_rates(ph_s)
edbm_dat.acid_channel()
edbm_dat.base_channel()
edbm_dat.salt_channel(Cm_bp_H, Cm_bp_OH)
Cna_s.append(edbm_dat.Ci_s_out[0])

# # Sum results

"Salt channel "
    #Concentration in salt channel 
Cbrine_out_t=sum(edbm_dat.Ci_s_out)
Cbrine_out=edbm_dat.Ci_s_out #mol/l
Cbrine_out_g=[Cbrine_out[0]*MW_Na, Cbrine_out[1]*MW_Cl, Cbrine_out[2]*MW_K, Cbrine_out[3]*MW_Mg, Cbrine_out[4]*MW_Ca, Cbrine_out[5]*MW_SO4] #g/l

     #Mass flow rate
M_s_out=edbm_dat.M_s_out_t*N_trip
print("Salt channel: Mass flow rate out is "+str(round(M_s_out,2))+"kg/hr")
    #Volumetric flow rate 
Q_s_out=edbm_dat.Q1_s_out*N_trip
print("Salt channel: Volumetric flow rate out is "+str(round(Q_s_out,2))+"l/hr")
print("Na concentration:"+str(round(Cbrine_out[0],2))+"M and "+str(round(Cbrine_out_g[0],2))+"g/l")
print("Cl concentration:"+str(round(Cbrine_out[1],2))+"M and "+str(round(Cbrine_out_g[1],2))+"g/l")
print("-----------------------------------------")

"Base channel "
    #Concentration in base channel 
Cb_out=edbm_dat.Ci_b_out[0:6]
Cb_out_g=[Cb_out[0]*MW_Na, Cb_out[1]*MW_Cl, Cb_out[2]*MW_K, Cb_out[3]*MW_Mg, Cb_out[4]*MW_Ca, Cb_out[5]*MW_SO4]

     #Mass flow rate
M_b_out=edbm_dat.M_b_out_t*N_trip
print("Base channel: Mass flow rate out is "+str(round(M_b_out,2))+"kg/hr")

     #Volumetric flow rate 
Q_b_out=edbm_dat.Q1_b_out*N_trip
print("Base channel: Volumetric flow rate out is "+str(round(Q_b_out,2))+"l/hr")

print("Na concentration "+str(round(Cb_out[0],2))+"M and "+str(round(Cb_out_g[0],2))+"g/l")
    #Conversion to solid 
M_NaOH_out=Q_b_out*edbm_dat.Ci_b_out[0]*constants.MW_NaOH/1000 #kg/hr 
print("-----------------------------------------")

"Acid channel" 
    #Concentration in acid channel 
Ca_out=edbm_dat.Ci_a_out
Ca_out=edbm_dat.Ci_a_out[0:6]
Ca_out_g=[Ca_out[0]*MW_Na, Ca_out[1]*MW_Cl, Ca_out[2]*MW_K, Ca_out[3]*MW_Mg, Ca_out[4]*MW_Ca, Ca_out[5]*MW_SO4]

    #Mass flow rate 
M_a_out=edbm_dat.M_a_out_t*N_trip
print("Acid channel: Mass flow rate out is "+str(round(M_a_out,2))+"kg/hr")

    #Volumetric flow rate 
Q_a_out=edbm_dat.Q1_a_out*N_trip
print("Acid channel: Volumetric flow rate out is "+str(round(Q_a_out,2))+"l/hr")
print("Cl concentration "+str(round(Ca_out[1],2))+"M and "+str(round(Ca_out_g[1],2))+"g/l")
print("-----------------------------------------")

    #Conversion to solid 
M_HCl_out=Q_a_out*constants.MW_HCl/1000 #kg/hr

#Calculate required amount of water for the operation mode 
Q_w_in=2*Q_in_edbm

#Calculate mass balance 
bal=(Q_in_edbm*d_in+2*Q_in_edbm)-M_a_out-M_s_out-M_b_out
print("Mass balance difference is "+str(round(bal,2)))
error_perc=abs(bal)/(Q_in_edbm*d_in+2*Q_in_edbm)*100
print("Balance error percentage is "+str(round(error_perc,2))+"%")

#Energy consumption 
V_ext=edbm_dat.V_ext #xternal 
#Calculate energy consumption for pumping 
Ppump=(edbm_dat.Q1_s_in*N_trip*dp+edbm_dat.Q1_a_in*N_trip*dp+edbm_dat.Q1_b_in*N_trip*dp)/1000/3600*1e5/npump #units: W -> l to m3 so /1000; bar to J 1e5N/m2*1J/m ; hr to 3660s  

#Calculate current efficiency 
Cb_in=[0]
CE=(Q_b_out)*(Cb_out[0]-Cb_in[0])*F/(3600*N_trip*I_d*A)*100 #%
print("Current efficiency is "+str(round(CE,2))+"%")
print("-----------------------------------------")
#Total energy consumption 
E_el_Edbm=V_ext*I_d*A/1000+Ppump/1000
SEC=(V_ext*I_d*A)/(Q_b_out*(edbm_dat.Ci_b_out[0]-edbm_dat.Ci_b_in[0])*constants.MW_NaOH)

print("Total electrical consumption for EDBM is " + str(round(E_el_Edbm,2))+ " KW")
print("Specific energy consumption is "+str(round(SEC,2))+"kwh/kg NaOH")

#%%
#Concentration of saline solution 
Q_out_b=Qd+Q_s_out
Cout_b=[]
for i in range(len(Cbrine_out_g)):
    Cout_b.append(Cbrine_out_g[i]*Q_s_out/Q_out_b+Sd_out[i]*Qd/Q_out_b)
Cout_b_t=sum(Cout_b)

#%%Indicators:
class indic:
    
    def __init__(self, tech, Qout, Qin, Qprod_1, prod_1_name, Qprod_2, prod_2_name, E_el, E_th, Cin, Cout, chem1,chem2):
        self.tech=tech
        self.Qout=Qout
        self.Qin=Qin
        self.E_el=E_el
        self.E_th=E_th
        self.Cin=Cin
        self.Cout=Cout
        self.Qprod_1=Qprod_1
        self.Qprod_2=Qprod_2
        self.prod_1_name=prod_1_name
        self.prod_2_name=prod_2_name
        self.chem=chem1+chem2
        
        
    def techn_indi(self):        
        if self.prod_1_name=="water": 
            self.Water_rr=self.Qprod_1/self.Qin*100 # %
            self.Qwater=self.Qprod_1
        elif self.prod_2_name=="water":
            self.Water_rr=self.Qprod_2/self.Qin*100 # %
            self.Qwater=self.Qprod_2
        else:
            self.Qwater=0
            self.Water_rr=0
            print("no water production by " + self.tech)
        
#%%Sumsummarize results  
tec1=indic("NF", Qconc, Qf, Qperm, "none", 0, "none", E_el_nf, 0, Ci_in, Cconc, QHCl_nf, Qantsc_nf)   
tec1.techn_indi()

tec2=indic("MF-PFR", Qout_2, Qconc, M_MgOH2_1,"Mg(OH)2", M_CaOH2, "Ca(OH)2", E_el_mfpf, 0, Cconc, Cout_mfpfr_g, QNAOH, QHCl)
tec2.techn_indi()

tec3=indic("ED", Qc, Qed_in, 0, "none", 0, "none",  Eel_t_ed, 0, Sc_i, Sd_out, 0,0)
tec3.techn_indi()

tec4=indic("EDBM", Q_s_out, Q_in_edbm, Q_b_out,"NaOH", Q_a_out, "HCl", E_el_Edbm, 0, Cin_edbm, Cbrine_out_g,0,0 )
tec4.techn_indi()

#
##list results 
tec_names=[tec1.tech,tec2.tech, tec3.tech,  tec4.tech]
E_el_all=[tec1.E_el,tec2.E_el, tec3.E_el, tec4.E_el]
E_th_all=[tec1.E_th,tec2.E_th, tec3.E_th,  tec4.E_th]
Cout_all=[tec1.Cout,tec2.Cout, tec3.Cout,  tec4.Cout]
Qout_all=[tec1.Qout, tec2.Qout, tec3.Qout,  tec4.Qout]
Qchem_all=[tec1.chem,tec2.chem,tec3.chem, tec4.chem]

#REQUIRED AMOUNT OF CHEMICALS 
Qnaoh_ex=Q_b_out-QNAOH
if Qnaoh_ex<0:
    Qnaoh_s=0
    Qnaoh_need=-Qnaoh_ex
else:
    Qnaoh_s=Qnaoh_ex
    Qnaoh_need=0
print("remaining amount of naoh is "+str(Qnaoh_ex) + "kg/hr")
Qhcl_ex=Q_a_out-QHCl
print("remaining amount of hcl is "+str(Qhcl_ex) + "kg/hr")
if Qhcl_ex<0:
    Qhcl_s=0
    Qhcl_need=-Qhcl_ex
else:
    Qhcl_s=Qhcl_ex
    Qhcl_need=0
#naoh need solid
Mnaoh_need=Qnaoh_need*constants.MW_NaOH/1000
Mhcl_need=Qhcl_need*constants.MW_HCl/1000

#
sum_el_en=sum(E_el_all)
sum_th_en=sum(E_th_all)
#%%
#Technical 
#T-1) quantity of water production 
Qw_tot=tec1.Qwater+tec2.Qwater+ tec3.Qwater+ tec4.Qwater
abs_Qw=Qw_tot-Q_w_in
print("absolute water production is "+str(abs_Qw))
#T-1b) Water recovery  
Qsw=Qsw
rec=Qw_tot/Qsw*100 #%
#T-2)energy performance: specific electrical energy consumption #kwh/kg of desalinated water 
SEC=sum(E_el_all)/Qw_tot
print("specific electrical consumption is " +str(SEC)+ " Kwh/kg of desalinated water")
Tot_el=sum(E_el_all) #kWh 
Tot_th=sum(E_th_all) #kWh

#T-3)Brine production -> kg of brine/ kg of seawater  
Qbrine_f=0
for i in range(len(tec_names)):
    if tec_names[i]=="EDBM" or tec_names[i]=="ED":
        if (sum(Cout_all[i])>sum(Csw)).any():
            Qbrine_f=Qbrine_f+Qout_all[i]
            print("for "+str(tec_names[i])+" brine concentration is "+str(sum(Cout_all[i])))
            print("for "+str(tec_names[i])+" brine production is "+str(Qout_all[i]))
        else:
            Qbrine_f=Qbrine_f+0
Q_br_prod=Qbrine_f/Qsw
print("brine production is "+str(Q_br_prod)+" kg of brine/kg of seawater")
#------------------------------------------------------------------------------
#%%
#Environmental 
#Env-1)Climate change impact ->Carbon dioxide emission kg co2 
emis=(sum(E_el_all)*CO2_el)+(sum(E_th_all)*CO2_st)
emis_t=((sum(E_el_all)*CO2_el)+(sum(E_th_all)*CO2_st))*constants.hr
print("Total specific Carbon dioxide emission "+str(emis_t)+" kg co2")
#Env-2) Resource utilization -> chemical consumption in kg of chemical/kg of seawater 
Chem_cons=(sum(Qchem_all))/Qsw #check units and convert ml/l to kg with densities 
print("Total chemical consumption is "+str(Chem_cons)+" kg of chemical/kg of seawater ")
#Env-3) Water footprint/ water use 
Qw_use=Q_w_in+Qnaoh_need+Qhcl_need

#%%
#Economic 
#Input data 
hr=24*300 #hours/year
lf= 20 #years
r=0.06 # interest rate
inf=0.02 # inflation rate 

#prices 
el_pr=0.253 #euro/kwh
s_pr=0 #euro/kwh
co2_em_c=23 #euro/ton co2 
nacl_pr=66/1000 # euro/kg 
hcl_pr=1040.0/180 #euro/l 1M solution 
w_pr=0.001 #euro/kg
na2so4_pr=140*0.83/1000 #euro/kg
mgoh2_pr=1000/1000 #euro/kg
caoh2_pr=125/1000#euro/kg
naoh_pr=6840/950#euro/L 1M NaOH solution  
antisc_pr=0.002 #euro/ml
cw_pr=0.118/1000#euro/kg
m_salt_pr=5/1000#euro/kg

#density
d_naoh=1.04 #kg/l for 1M solution 
d_hcl=1.0585#kg/l for 1M solution 

#Assumptions for CAPEX 
inst_percent=0.25
buildings_percent=0.2
land_percent=0.06
indirect_c_percent=0.15
workinf_c_percent=0.2
capex_assumptions=[inst_percent,buildings_percent, land_percent, indirect_c_percent, workinf_c_percent]


#Assumptions for  OPEX
main_c_percent=0.03  # % of the fixed-capital investment
oper_sup_c_percent=0.05  # % of maintenance 
oper_lab_c_percent=0.15    # % of annual OPEX
super_c_percent=0.15 # % of operating labor 
lab_c_percent=0.15 # % of operating labor 
pat_c_percent=0.03 # % of annual OPEX
fix_char_percent=0.05 # % of annual OPEX
over_c_percent=0.05 # %of annual OPEX
norm_factor =(1 -oper_lab_c_percent-oper_lab_c_percent*super_c_percent- oper_lab_c_percent*lab_c_percent-pat_c_percent-fix_char_percent-over_c_percent)
economic_assumptions=[main_c_percent,oper_sup_c_percent, oper_lab_c_percent, super_c_percent, lab_c_percent, pat_c_percent, 
                      fix_char_percent, over_c_percent, norm_factor ]

# Initialize values
CAPEX=0
OPEX=0
CO2_c=0

 # Create list with capital cost for each unit.
capex_list=[]
 # Create list with operating for each unit.
opex_list=[]

#summarise results for utilities 
el_conc=E_el_all
s_conc=E_th_all
chem1_conc=[Qantsc_nf,0,Mnaoh_need,0]
chem1_pr=[0.002,0,constants.naoh_pr_s,0]
chem2_conc=[0,0,Mhcl_need,0]
chem2_pr=[0.002,0,constants.hcl_pr_s,0]
wat_conc=[0,0, Qnaoh_need, Q_w_in]
cw_conc=[0,0, 0, 0]

# Calculation of scale-up equipment cost 
# Import equipment cost for reference scenario from constants function
eq_c=[constants.eq_c_nf, constants.eq_c_ed, constants.eq_c_mfpfr, constants.eq_c_edbm]  
    # Capacity of reference scenario
Mf_basic_sc=[]
Mf_basic_sc=[constants.Mf_basic_sc[0],constants.Mf_basic_sc[7], constants.Mf_basic_sc[3], constants.Mf_basic_sc[6]]

    # Calculation of the new equipment cost
Mf_sce=[Qsw,   Qed_in, Qin_mfpfr, Q_in_edbm]
for i in range(len(eq_c)):
    if Mf_basic_sc[i]!=Mf_sce[i]:
        eq_c[i]=scaleup.scaleup_eq(eq_c[i],Mf_basic_sc[i],Mf_sce[i],tec_names[i])
        
#Cost calculation 
for i in range(len(eq_c)):
    total_econom=econom(eq_c[i], el_conc[i], s_conc[i], chem1_conc[i], chem1_pr[i],chem2_conc[i], chem2_pr[i], cw_conc[i], wat_conc[i])
    total_econom.capex_calc(capex_assumptions)
    total_econom.opex_calc(hr, el_pr, s_pr, cw_pr, w_pr, economic_assumptions)
    CAPEX=total_econom.t_capital_inv
    OPEX=total_econom.opex
    opex_list.append(total_econom.opex)
    capex_list.append(total_econom.t_capital_inv)    
print("Total operating cost (OPEX) is "+str(OPEX)+ " Euro/year") 
print("Total investment cost (CAPEX) of system is " + str(round(CAPEX))+ " Euro")    
print("-----------------------------------------")

#amortisation factor 
a=(constants.r*(1+constants.r)**constants.lf)/((1+constants.r)**constants.lf-1)

"""Revenue calculation"""
#Input data 
reve_t=0
reve_list=[]
prd=[Qw_tot, M_MgOH2_1, Q_b_out, Q_a_out]    
prd_name= ["Water",  "Mg(OH)2", "NaOH", "HCl"]   

#Revenue calculation
for i in range(len(prd)):
    rev_calc=revenue(prd[i], prd_name[i])    
    rev_calc.rev(hr, w_pr, nacl_pr, mgoh2_pr,na2so4_pr, naoh_pr, hcl_pr)
    print("Revenues from selling product " + prd_name[i]+" are " + str(round(rev_calc.rev_prd,2))+" Euro/year")
    reve_t = reve_t+rev_calc.rev_prd
    reve_list.append(rev_calc.rev_prd)
    
#%%Present results
#Summary tables 
    #Table C: Ion concentration 
sum_table_C={'F1: Seawater': Ci_in,
           'F2: NF permeate':Cperm, 
           'F3: NF concentrate': Cconc,
           'F4: ED saline solution': Sd_out, 
           'F5: ED to EDBM': Sc_out,
           'F6: MF-PFR out': Cout_mfpfr_g, 
           'F7: MgOH2': [0,0,0,0,0,0],
           'F8: CaOH2': [0,0,0,0,0,0],
           'F9: EDBM: ACID': Ca_out_g[0:6],
           'F10: EDBM: BASE ':Cb_out_g[0:6],
           'F11: EDBM: SALT':Cbrine_out_g[0:6],
           }
sum_table_C=pd.DataFrame(sum_table_C)
print(sum_table_C)

    #Table D: Density
sum_table_d={'F1: Seawater': density_calc(25,sum(Ci_in))/1000,
           'F2: NF permeate':density_calc(25,sum(Cperm))/1000, 
           'F3: NF concentrate': density_calc(25,sum(Cconc))/1000,
           'F4: ED saline solution': density_calc(25,sum(Sd_out))/1000, 
           'F5: ED to EDBM': density_calc(25,sum(Sc_out))/1000,
           'F6: MF-PFR out':density_calc(25,sum( Cout_mfpfr_g))/1000, 
           'F7: MgOH2': 1,
           'F8: CaOH2': 1,
           'F9: EDBM: ACID': density_calc(25,sum(Ca_out_g[0:6]))/1000,
           'F10: EDBM: BASE ': density_calc(25,sum(Cb_out_g[0:6]))/1000,
           'F11: EDBM: SALT': density_calc(25,sum(Cbrine_out_g[0:6]))/1000,
           }
sum_table_d=pd.DataFrame(sum_table_d, index=['density'])
print(sum_table_d)

        #Table F: Flow rates  
sum_table_f={'F1: Seawater': round(Qf,2),
           'F2: NF permeate':round(Qperm,2), 
           'F3: NF concentrate': round(Qconc,2),
           'F4: ED diluate': round(  Md,2), 
           'F5: ED to EDBM': round(Mc,2),
           'F6: MF-PFR out': round(M_mfpfr_out,2), 
           'F7: MgOH2': round(M_MgOH2_1,2),
           'F8: CaOH2': round(M_CaOH2,2),
           'F9: EDBM: ACID':round(Q_a_out,2),
           'F10: EDBM: BASE ':round(Q_b_out,2),
           'F11: EDBM: SALT':round(Q_s_out,2),
           'F12: EDBM: water in':round(Q_w_in,2),
           'F13: MF-PFR NaOH': round(QNAOH,2),
           'F14: MF-PFR HCl': round(QHCl,2),
           'F15: NF antiscalant':round(Qantsc_nf,2)
           }
sum_table_f=pd.DataFrame(sum_table_f, index=['flow rate'])
print(sum_table_f)

#sankey diagram 
import plotly.graph_objects as go
from plotly.offline import plot

#Figure 1: Sankey diagram for Example: Mass flow rates
fig = go.Figure(data=[go.Sankey(
    node = dict(
      pad = 15,
      thickness = 20,
      line = dict(color = "black", width = 0.5),
      label = ["Seawater", "Water", "NaCl",  "NF", "ED", "MF-PFR", "EDBM", "HCl", "NaOH", "Mg(OH)\u2082", "Ca(OH)\u2082", "Utilities", "Saline solution"],
      color = "blue"
    ),
    link = dict(
      source = [0,3,3,4,4,5,5,5,6,6,6,11,11], 
      target = [3,4,5,6,12,6,9,10,7,8,12,5,6],
      value = [Qsw, Qperm, Qconc,Mc, Md,M_mfpfr_out, M_MgOH2_1,M_CaOH2,Q_a_out, Q_b_out,Q_s_out, QNAOH+QHCl,Q_w_in ]
  ))])
color_for_nodes = ["lightsteelblue","darkcyan","maroon", "midnightblue", "midnightblue", "midnightblue", "midnightblue", "gold", "gold", "gold", "gold", "gold","grey"]
fig.update_traces(node_color = color_for_nodes)
fig.update_layout(title_text="Sankey diagram for Example: Mass flow rates ", font_size=15)
fig.show()
plot(fig)

#%% Results to excel 
# Create dataframes 
dfel=pd.DataFrame(E_el_all ,tec_names)
dfeth=pd.DataFrame(E_th_all, tec_names)
dfind=(Qw_tot, abs_Qw, rec, Tot_el, Tot_th, emis_t, Q_w_in,  OPEX, CAPEX)
dfprod=(Qw_tot, M_MgOH2_1, M_CaOH2,  M_b_out, M_a_out)

dfprodn=("Water (kg/hr)",  "Mg(OH)2(kg/hr)", "Ca(OH)2(kg/hr)",  "1M NaOH (kg/hr)","1M HCl(kg/hr)")
ind=np.array(["Water production", "absolute water production", "water recovery","Total electrical consumption", "Total thermal energy consumption", "Carbon dioxide emission kg co2/year",
              "Water footprint","OPEX", "CAPEX"])
units=pd.DataFrame(["kg/hr", "kg/hr", "%","KWh", "KWh"," kg co2/year","kg/hr","€/year", "€"])
dfind_t=pd.DataFrame(data=dfind, index=ind)
dfprodn=pd.DataFrame(data=dfprod, index=dfprodn)

  # Write results in excel document
with pd.ExcelWriter('results_example.xlsx') as writer:
    sum_table_f.to_excel(writer,sheet_name="example")
    sum_table_C.to_excel(writer,sheet_name="example", startcol=0, startrow=6, header=True)
    sum_table_d.to_excel(writer,sheet_name="example", startcol=0, startrow=2, header=True)
    dfel.to_excel(writer,sheet_name="example", startcol=0, startrow=14, header=True)
    dfeth.to_excel(writer,sheet_name="example", startcol=2, startrow=14, header=True)
    dfprodn.to_excel(writer,sheet_name="example", startcol=0, startrow=23, header=True)
    #dfprod.to_excel(writer,sheet_name="water_mining_scenario", startcol=1, startrow=24, header=True)
    dfind_t.to_excel(writer,sheet_name="indicators")
    units.to_excel(writer,sheet_name="indicators",startcol=2, startrow=0, header=True)


from desalsim.density_calc import density_calc 
#%%calculations 
class MEDCalculator:
    """
    A class used to represent Mass Balance for Multi-Effect Distillation 

    ...

    Attributes
    ----------
    Qf : float
        Flow rate (m^3/s).
    CNa_in, CCl_in, CK_in, CMg_in, CCa_in, CSO4_in : float
        Initial concentrations of various ions (g/l).

    Methods
    -------
    salinity_calc():
        Calculates the inflow salinity.
        
    mass_balance_med():
        Performs mass balance calculations.
        
    temperature_calc(T_s, T_N, T_cw_in, DT_loss):
        Calculates temperature-related parameters.
        
    performance_parameters():
        Calculates performance parameters.
        
    output_concentration():
        Calculates the concentrations in the output.
 
    """
    
    def __init__(self,Qf, Mf_med, CNa_in, CCl_in, CK_in, CMg_in, CCa_in, CSO4_in, T_in):
        #Initialize class attributes
        self.Qf = Qf
        self.Mf_med=Mf_med
        self.CNa_in = CNa_in
        self.CCl_in = CCl_in
        self.CK_in = CK_in
        self.CMg_in = CMg_in
        self.CCa_in = CCa_in
        self.CSO4_in = CSO4_in
        self.cons=[self.CNa_in,self.CCl_in,self.CK_in,self.CMg_in,self.CCa_in,self.CSO4_in]
        self.T_in=T_in
        self.d=density_calc(self.T_in, sum(self.cons))


    def salinity_calc(self):
        # Calculate inflow salinity
        self.salinity_in=sum(self.cons)
        self.xf=self.salinity_in/self.d*1000
        self.Mf=self.Mf_med/3600 #kg/s
        
        
    def mass_balance_med(self, Cb_out):
        # Perform mass balance calculations
        #Brine Concentration leaving effect n 
        self.xn=Cb_out
        
        #Calculate concentration factor 
        self.conc_f=self.xn/self.xf
        
        #Calculate brine flow rate of leaving effect n
        self.Bn=self.xf*self.Mf/self.xn #kg/s
        self.Qb=self.Bn*3600 #kg/hr
        
        #Calculate total distillate flow rate
        self.Mdist=self.Mf-self.Bn #kg/s
        self.Qdist=self.Mdist*3600 #kg/hr
        
        #Calculate distillate flow produces for each effect
        self.D1=self.Mdist/(1+self.lhv1/self.lhv2) #kg/s
        self.D2=self.D1*self.lhv1/self.lhv2 #kg/s

        
        
    def temperature_calc(self, DT_loss, T_N, T_s):
        # Calculate temperature-related parameters
        T3=69
        #Transfer coefficients of the effects,
        self.U1= 1.9695+0.012057*T3-0.000085989*T3**2+0.00000025651*T3**3
        self.U2=0.95*self.U1
        
        #Temperature drop in each effect 
        self.DT_t=T_s-T_N #oC
        self.DT1=self.DT_t/(self.U1*(1/self.U1+1/self.U2)) #oC
        self.DT2=self.DT1*self.U1/self.U2 #oC
        
        #Temperature profile of each effect 
        self.T1=T_s-self.DT1 #oC
        self.T2=self.T1-self.DT2 #oC
        
        #Latent heat of first effect 
        self.lhv1=2499.5698-0.9156*(self.T1-DT_loss)-0.048343*(self.T1-DT_loss)**2
        
        #Latent heat of second effect
        self.lhv2=2626.1
    
    def performance_parameters(self, lh_s,  T_cw_in):
        # Calculate performance parameters
        #Steam mass flow rate
        self.Ms=self.D1*self.lhv1/lh_s #kg/s
        
        #Gain Output Ratio
        self.GOR=self.Mdist/self.Ms
        
        #Condenser thermal load (Qc) kj/s ot kw
        self.Qc=self.D2*self.lhv2 
        
        #thermal load in the first effect (Q1) kw
        self.Q1= self.Ms*lh_s
        
        #Cooling water flow rate kg/s
        Cp_w=4182 # specific heat capacity of water (j/kgC)
        self.QCW= self.D2*self.lhv2*1000/(Cp_w*(self.T_in-T_cw_in))-self.Mf
        
        #Performance ratio (PR) 
        self.PR= self.Mdist/self.Mf
        
        #Condenser thermal load
        cp_sol=4184 # specific heat capacity of solution (j/kgC)
        self.Qsen=self.Mf*cp_sol*(self.T_in-25)
        
        #Total thermal load
        self.Q_Tot=self.Q1+self.Qsen/1000

    def output_concentration(self):
        # Calculate concentrations in the output, g/l
        self.CNa_out=self.CNa_in*self.Mf/self.Bn 
        self.CCl_out=self.CCl_in*self.Mf/self.Bn 
        self.CK_out=self.CK_in*self.Mf/self.Bn 
        self.CMg_out=self.CMg_in*self.Mf/self.Bn 
        self.CCa_out=self.CCa_in*self.Mf/self.Bn  
        self.CSO4_out=self.CSO4_in*self.Mf/self.Bn  

 #%%
# #Example usage

#Feed concentration
components = ['Na', 'Cl', 'K', 'Mg', 'Ca', 'SO4']
Cin_med = [10.36, 15.39, 0.36, 0.028, 0.02, 0.07]

#Feed flow rate 
Qf_med =1000 #l/hr

#input conditions
T=20
#feed flow density 
d=density_calc(T, sum(Cin_med)) 
Mf_med=Qf_med*d/1000 #Mass flow rate (units: kg/hr)

#assumptions:
T_in=40 #(oC)
T_N=45 #Temperature in the last effect (oC)
N=2 #Number of effects (-)
Cp_w=4182 # specific heat capacity of water (j/kgC)
cp_sol=4184 # specific heat capacity of solution (j/kgC)
T_cw_in=25 #intake cooling water temperature (oC)
T_cw_out=35 #out cooling water temperature (oC)
T_s=70 #steam temperature oC
DT_loss=1 #temperature difference (oC)
T3=69
dp=0.1  # pressure drop (units: bar)
dp_slurry=1 # pressure drop (units: bar)
npump=0.8 #pump efficiency (units: -)
Cb_out=200 #Brine Concentration leaving effect n (unit: g/l)
Xr=5.5 # brine circulation flow rate (units: -)

#latent heat of motive steam:
if T_s<=55:
    lh_s=2370
elif T_s>55 and T_s<=60:
    lh_s=2358
elif (T_s>60) and (T_s<=65):
    lh_s=2345
elif (T_s>65) and (T_s<=70):
    lh_s=2333
elif (T_s>70) and (T_s<=75):
    lh_s=2321


# Create an instance of the MEDCalculator class
med_dat = MEDCalculator(Qf_med, Mf_med, Cin_med[0], Cin_med[1], Cin_med[2], Cin_med[3], Cin_med[4], Cin_med[5], T_in)

# Call methods to perform calculations
med_dat.salinity_calc()
med_dat.temperature_calc(DT_loss, T_N, T_s)
med_dat.mass_balance_med(Cb_out)
med_dat.performance_parameters(lh_s,  T_cw_in)
med_dat.output_concentration()


# Sum results
#Concentration of brine stream 
Cconc_med = [med_dat.CNa_out, med_dat.CCl_out, med_dat.CK_out, med_dat.CMg_out, med_dat.CCa_out, med_dat.CSO4_out]

#Brine flow rate 
Qout_med=med_dat.Qb

#Distillate water flow rate 
Qprod_med=med_dat.Qdist

#Calculate circulation flow rate 
Qr=Xr*Qf_med

print("Brine flow rate: "+ str(round(Qout_med,2))+"kg/hr")
print("Total distillate flow rate: "+ str(round(Qprod_med,2))+"kg/hr") 
# Calculate density for output concentration
d_b = density_calc(45, sum(Cconc_med))

print("Sum of output concentrations: " + str(round(sum(Cconc_med),2))+"g/l")
print("-----------------------------------------")

# Calculate mass balances
bal_t=0
for i in range(len(Cconc_med)):
    bal_i = (Cin_med[i] * Qf_med /  1000) - (med_dat.Qb / (d_b / 1000) * Cconc_med[i]/1000)
    bal_t=bal_t+bal_i
    print("Mass balance for " + components[i] + ": " + str(round(bal_i,2)))
error_bal=bal_t/(sum(Cin_med)* Qf_med /  1000)*100
print("Ion balance error percentage is "+str(round(error_bal,2))+"%")
print("-----------------------------------------")

# Calculate energy consumption
E_el_med = ((Qf_med * 3.5 + med_dat.QCW * 3600 * 2 + (Qr + med_dat.Qb) * 3.5 + med_dat.Qdist * 1) / (1000 * npump)) * 1e5 / 3600 / 1000  # kWh
print("Electrical energy consumption: " + str(round(E_el_med,2)) + " kWh")

SEC_el = E_el_med / (Qf_med / d)  # kWh/m3 feed
print("Specific energy consumption (electrical) per m3 feed: " + str(round(SEC_el,2)) + " kWh/m3")

SEC_el_prod = E_el_med / (med_dat.Qdist / 1000)  # kWh/m3 dist water
print("Specific energy consumption (electrical) per m3 product (distilled water): " + str(round(SEC_el_prod,2)) + " kWh/m3")
print("-----------------------------------------")

E_th_med = med_dat.Q_Tot
print("Total thermal energy consumption: " + str(round(E_th_med,2)) + " kW")

SEC_th = E_th_med / (Qf_med / d)  # kWh_th/m3
print("Specific energy consumption (thermal) per m3 feed: " + str(round(SEC_th,2)) + " kWh_th/m3")
print("-----------------------------------------")

#Calculate required cooling water 
Qcw = med_dat.QCW * 3600 #units: kg/hr
print("Cooling water flow rate: " + str(round(Qcw,2)) + " kg/hr")
print("-----------------------------------------")

# Chemical consumption
Cchem = 0  # Placeholder for chemical consumption, update as needed

# Print the results
print("Total Chemical Consumption: " + str(round(Cchem,2))+"kg/hr")


import unittest
from desalsim.med_unit_f import MEDCalculator, density_calc

class TestMEDCalculator(unittest.TestCase):
    def setUp(self):
        # Define the input parameters for testing
        Cin_med = [10.36, 15.39, 0.36, 0.028, 0.02, 0.07]
        Qf_med = 1000
        Mf_med = 174.475 # Assuming a density of 1 g/ml
        
        # Add any other necessary parameters for initialization
        T_in = 40
        
        # Create an instance of the MEDCalculator class with the defined parameters
        self.med_dat = MEDCalculator(Qf_med, Mf_med, Cin_med[0], Cin_med[1], Cin_med[2], Cin_med[3], Cin_med[4], Cin_med[5], T_in)

    def test_salinity_calc(self):
        # Call the salinity_calc method
        self.med_dat.salinity_calc()
        
        # Define the expected values
        expected_salinity_in = sum([10.36, 15.39, 0.36, 0.028, 0.02, 0.07])
        expected_xf = expected_salinity_in / density_calc(40, expected_salinity_in) * 1000
        expected_Mf = 174.475 / 3600 # Converting from kg/hr to kg/s
        
        # Assert the equality of the calculated values and expected values
        self.assertAlmostEqual(self.med_dat.salinity_in, expected_salinity_in, delta=expected_salinity_in*0.01)
        self.assertAlmostEqual(self.med_dat.xf, expected_xf, delta=expected_xf*0.01)
        self.assertAlmostEqual(self.med_dat.Mf, expected_Mf, delta=expected_Mf*0.01)

    # Add more test methods to cover other methods in the MEDCalculator class

if __name__ == '__main__':
    unittest.main()



# __init__.py
from desalsim import density_calc
from desalsim import constants

from desalsim.nanofiltration_unit_f import NFMass
from desalsim.nanofiltration_unit_f import OsmoticPressure
from desalsim.nanofiltration_unit_f import NfEnergy

from desalsim.med_unit_f import MEDCalculator

from desalsim.thermal_cryst_f import thermal_calc
from desalsim.thermal_cryst_f import conc_cal
from desalsim.thermal_cryst_f import calculate_energy

from desalsim.mfpfr_unit_f import MFPFRCALC
from desalsim.mfpfr_unit_f import HClAddition
from desalsim.mfpfr_unit_f import energycons

from desalsim.ed_unit_f import ElectrodialysisCalc

from desalsim.edbm_unit_f import EDBMCalc

from desalsim.economic_f import econom
from desalsim.economic_f import revenue









import unittest
from desalsim.mfpfr_unit_f import MFPFRCALC, HClAddition

class TestMFPFRCALC(unittest.TestCase):
    def setUp(self):
        # Define sample input data for initialization
        Qin = 1000
        Cin_mfpfr = [17.3, 38.6, 0.6, 4.3, 1.2, 9.9]
        C_NaOH_1 = 1
        C_NaOH_2 = 1
        conv_1 = 95
        conv_2 = 93
        
        # Create an instance of the MFPFRCALC class
        self.mfpfr_calc = MFPFRCALC(Qin, Cin_mfpfr, C_NaOH_1, C_NaOH_2, conv_1, conv_2)
        self.result = self.mfpfr_calc.calc_step1(5.61 * 0.000000000001, 2.34)
        
        
    def test_calc_step1(self):
        # Test the calc_step1 method
        
        # Call the calc_step1 method
        self.result = self.mfpfr_calc.calc_step1(5.61 * 0.000000000001, 2.34)
        
        # Assert the correctness of the output
        expected_value_QMgOH=9.80
        expected_value_QNaOH=336.14
        self.assertAlmostEqual(round(self.result[1],2), expected_value_QNaOH, delta=expected_value_QNaOH*0.01)
        self.assertAlmostEqual(round(self.result[2],2), expected_value_QMgOH, delta=expected_value_QMgOH*0.01)
        self.assertAlmostEqual(round(self.result[3],2), 1331.96, places=2)


    def test_calc_step2(self):
        # Test the calc_step2 method
        
        # Call the calc_step2 method
        self.result_2 = self.mfpfr_calc.calc_step2(2.34, 2.211)
        
        # Assert the correctness of the output

        expected_value_QNaOH_2=73.38
        self.assertAlmostEqual(round(self.result_2[1],2), expected_value_QNaOH_2, delta=expected_value_QNaOH_2*0.01)
        self.assertAlmostEqual(round(self.result_2[4],2), 1526.61, delta=1526.61*0.01)


class TestHClAddition(unittest.TestCase):
    def setUp(self):
        # Define sample input data for initialization
        Qout_2 = 1526.26
        Cout_all_m = [0.8, 0.7, 0.01, 0.0, 0.0, 0.07]
        MW_Cl = 35.45
        ph_2 = 13
        HCl_conc = 1
        
        # Create an instance of the HClAddition class
        self.hcl_addition = HClAddition(Qout_2, Cout_all_m, MW_Cl, ph_2, HCl_conc)

    def test_calculate_HCl_addition(self):
        # Test the calculate_HCl_addition method
        
        # Define additional input data
        Cout_mfpfr_g = [1, 2, 3, 4, 5, 6]
        
        # Call the calculate_HCl_addition method
        result = self.hcl_addition.calculate_HCl_addition(Cout_mfpfr_g)
        
        # Assert the correctness of the output
        expected_value_HCl=152.66
        self.assertAlmostEqual(round(result[0],2), expected_value_HCl, delta=expected_value_HCl*0.01)
        # Add more assertions as needed

if __name__ == '__main__':
    unittest.main()



#Density calculation based on the stream salinity and temperature
"""
    Calculate the density of water based on its temperature and salinity.

    This function uses empirical equations to calculate the density of seawater 
    based on temperature (T) and salinity (S) using the UNESCO 1983 (EOS 80) formula.

    Parameters:
    -----------
    T : float
        Temperature of the water in degrees Celsius (°C).
    S : float
        Salinity of the water in grams per kilogram (g/kg).

    Returns:
    --------
    float
        The calculated density of the seawater in kilograms per cubic meter (kg/m³).
    
    Notes:
    ------
    The calculation is based on the following empirical formulas:
    - A: Polynomial coefficients for temperature dependence
    - B: Polynomial coefficients for salinity dependence
    - C: Second-order polynomial term for salinity dependence
    - rho: Pure water density at the given temperature
    - rhos: Final seawater density considering both temperature and salinity
"""

def density_calc(T, S):
    A= 8.24493E-1 - 4.0899E-3*T + 7.6438E-5*T**2 -8.2467E-7*T**3 + 5.3675E-9*T**4
    B = -5.724E-3 + 1.0227E-4*T - 1.6546E-6*T**2
    C = 4.8314E-4
    rho = 1000*(1 -(T+288.9414)/(508929.2*(T+68.12963))*(T-3.9863)**2)
    rhos = rho + A*S + B*S**(3/2) + C*S**2
    return rhos


import numpy as np
import math
from desalsim.density_calc import density_calc 
from desalsim import constants 
from desalsim import scaleup

#%% Calculations 
"""
A class used to represent Mass Balance for Electrodialysis

...

Attributes
----------
    MWs : float
        Molecular weight of NaCl (g/mol)
    MWw : float
        Molecular weight of water (g/mol)
    R : float
        Resistance of rinse stream (ohm)
    Rp : float
        Resistance of polarization (ohm)
    A : float
        Active area of cell-pair (m^2)
    F : float
        Faraday constant (C/mol)
    T : float
        Temperature in Kelvin
    dp : float
        Parameter dp
    npump : float
        Pump efficiency
    rho_w : float
        Density of water (kg/m^3)
    D : float
        Diffusion coefficient (m^2/s)
    tcu : float
        tcu parameter
    veloc : float
        Velocity (m^2/s)
    h : float
        Height (mm)
    Sh : float
        Sh parameter
    Mem_eff : float
        Membrane efficiency
    Ncp : integer
        Number of cell-pairs
    Qed_in : float
        Inlet flow rate (l/h)
    Qed_in_c : float
        Concentrate inlet flow rate (l/h)
    Qed_in_d : float
        Dilute inlet flow rate (l/h)


Methods
-------
    Ts_cp(S): 
        Transport number for salt in concentrate compartment
    Tw_cp(Sc, Sd): 
        Transport number for water in concentrate compartment
    Ls_cp(Sc, Sd): 
        Permeability for salt in concentrate compartment
    Lw_cp(S): 
        Permeability for water in concentrate compartment
    p_osmo(S): 
        Osmotic pressure in concentrate compartment
    p_osmo2(S): 
        Alternative osmotic pressure calculation
    dC(Ts_cp): 
        Change in concentration
"""
class ElectrodialysisCalc:
    def __init__(self):
        pass

    def Ts_cp(S):
        """Transport number for salt in concentrate compartment"""
        Ts_cp=-4e-6*S**2 + 4e-5*S + 0.96
        return Ts_cp
    
    def Tw_cp(Sc,Sd):
        """Transport number for water in concentrate compartment"""
        return -4e-5*Sc**2 - 1.9e-2*Sd + 11.2
    
    def Ls_cp(Sc, Sd):
        """Permeability for salt in concentrate compartment"""
        return min(2e-12*Sd**2 - 3e-10*Sd + 6e-8, 2e-12*Sc**2 - 3e-10*Sc + 6e-8)
    
    def Lw_cp(S):
        """Permeability for water in concentrate compartment"""
        return 5*S**(-0.416)
    
    
    def p_osmo(S, T, MWs): 
        """calculation for Osmotic pressure in concentrate compartment"""
        C1=S/MWs*constants.MW_Na/constants.MW_Na
        C2=S/MWs*constants.MW_cl/constants.MW_cl
        sum_Ci=sum([C1, C2])
        p_osmo=0.0831446261815324*sum_Ci*T
        return p_osmo
    
    def dC(Ts_cp, tcu, D, Ij,  h, Sh ):
        """Change in concentration"""
        F = 96485.3329  # Faraday constant (C/mol)
        Tcu=(Ts_cp+1)/2
        dC=-(Tcu-tcu)/D*(Ij/F)*(2*h/1000/Sh)
        return dC

#%%
#user example 
# Constants
MWs = 58.44  # Molecular weight of NaCl (g/mol)
MWw=18.01528# Molecular weight of water (g/mol)

#Assumptions 
R = 0.002  # Resistance of rinse stream (ohm)
Rp = 0.015  # Resistance of polarization (ohm)
A = 1.1  # Active area of cell-pair (m^2)
F = 96485.3329  # Faraday constant (C/mol)
T=20+273.15 #Inlet/operating temperature (K)
dp=1 #pressure drop (units: bar)
npump=0.8 #pump efficiency (units: -)
rho_w=1000 # water desnity kg/m3

#model parameters 
D=1.61e-9 #Diffusion coefficient (m^2/s)
tcu=0.5
veloc=8.9e-7 #m2/s
h=0.5 #mm
Sh=18

#Input data 
Sc_i = 43.39  # Salinity at concentrate inlet (g/kg)
Sc_o = 200  # Salinity at concentrate outlet (g/kg)
Sd_o=20 # Salinity at doluate outlet (g/kg)
Sd_i=Sc_i # Salinity at diluate inlet (g/kg)
N = 50 # Number of computational cells per cell-pair
Vcp = 8  # Applied voltage (V)
Vel=2.1 #Voltage across the electrodes 
Ij=400 #Am2
Mem_eff=0.64 #Membrane efficiency
Ncp=1 #Number of identical parallel cell-pairs 

#Feed flow rate L/h
d_in_ed=(density_calc(T-273, Sc_i)/1000)
Qed_in=1000/d_in_ed
Qed_in_c=Qed_in/17/Ncp
Qed_in_d=Qed_in*16/17/Ncp

# Effective cell-pair area (m^2)
Acp =scaleup.scaleup(24, 1000, Qed_in) 
Acp_tot=Acp

# Initializations
Sc = np.zeros(N)
Sd = np.zeros(N)
Ns_c = np.zeros(N)
Ns_d = np.zeros(N)
Nw_c = np.zeros(N)
Nw_d = np.zeros(N)
Js = np.zeros(N)
Jw = np.zeros(N)
Mw_in_d_l=np.zeros(N)
Ms_d=np.zeros(N)
Mw_d=np.zeros(N)
M_d=np.zeros(N)
M_c=np.zeros(N)
Q_c=np.zeros(N)
Q_d=np.zeros(N)

# Set initial values salt stream 
Sc[0] = Sc_i
Ns_c[0]=Qed_in_c*d_in_ed*Sc[0]/MWs #mol/hr
Ms_in_c=Qed_in_c*d_in_ed*Sc_i/1000 #kg salr/hr
Mw_in_c=Qed_in_c*d_in_ed-Ms_in_c #kg water/hr
Nw_c[0]=Mw_in_c*1000/MWw #mol/hr
M_c[0]=Mw_in_c+Ms_in_c
Q_c[0]=Qed_in_c


# Set initial values diluate stream 
Csw= [17.17, 25.47, 0.57, 0.04, 0.03, 0.10]
Sd[0] = Sc_i #g/kg 
Ns_d[0]=Qed_in_d*d_in_ed*Sd[0]/(MWs) #mol/s
Ms_in_d=Qed_in_d*d_in_ed*Sd[0]/1000 #kg salr/hr
Mw_in_d=Qed_in_d*d_in_ed-Ms_in_d #kg water/hr
Nw_d[0]=Mw_in_d*1000/MWw #mol/hr
Mw_in_d_l[0]=Mw_in_d
Ms_d[0]=Ms_in_d
Mw_d[0]=Mw_in_d
M_d[0]=Mw_in_d+Ms_in_d
Q_d[0]=Qed_in_d

# Initialize the Acp_tot array
Acp_tot_j=Acp_tot/N
ed_em = ElectrodialysisCalc()

# Iterate over cells
for j in range(1, N):
    #Calculate salinity change 
    concentration_diff = Sc[j - 1] - Sd[j - 1]
    Sc[j] = Sc[j - 1] + (Sc_o - Sc_i) / (N - 1)
    Sd[j] = Sd[j - 1] + (Sd_o - Sd_i) / (N - 1)
    
    #Calculate net salt flux 
    Js[j] = (ElectrodialysisCalc.Ts_cp(Sd[j - 1]) * Ij / F - 
             (ElectrodialysisCalc.Ls_cp(Sc[j - 1], Sd[j - 1])) * concentration_diff)
    #calculate net water flux 
    Jw[j] = (ElectrodialysisCalc.Tw_cp(Sc[j - 1], Sd[j - 1]) * Ij / F +
             ElectrodialysisCalc.Lw_cp(Sc[j - 1]) * (ElectrodialysisCalc.p_osmo(Sc[j - 1], T, MWs) - 
                                                      ElectrodialysisCalc.p_osmo(Sd[j - 1], T, MWs)))
    
    #Calculate he total concentrate and dilute molar flow rates
    Ns_c[j] = Ns_c[j - 1] + Acp_tot_j * Js[j]
    Ns_d[j] = Ns_d[j - 1] - Acp_tot_j * Js[j]

    Nw_c[j] = Nw_c[j - 1] + Acp_tot_j * Jw[j]
    Nw_d[j] = Nw_d[j - 1] - Acp_tot_j * Jw[j]

    # Update the flow rates of the concentrate and dilute streams
    Q_c[j] = Nw_c[j] * MWw / (rho_w * (1 - Sc[j] / 1000))
    Q_d[j] = Nw_d[j] * MWw / (rho_w * (1 - Sd[j] / 1000))





Cc_na_f=Sc[N-1]/MWs*constants.MW_Na
Cc_cl_f=Sc[N-1]/MWs*constants.MW_cl
Sc_out=[Cc_na_f, Cc_cl_f]

#Calculate the concnetrate stream flow rate 
Mc=(Ns_c[N-1]*MWs/1000+Nw_c[N-1]*MWw/1000) #(kg/hr)
print("Mass flowrate concentrate stream is "+str(round(Mc,2))+ " kg/hr")

dc_out=density_calc(T-273, Sc[N-1])/1000 #(kg/l)
Qc=Mc/dc_out #concnetrate stream volume flow rate (l/hr)
i=2

for i in range(2,len(Csw)):
    Sc_out.append(Csw[i]*Qed_in_c/Qc)

print("Volume flowrate concentrate stream is "+str(round(Qc,2))+" l/hr")
print("The total effluent concentration concentrate stream  is " + str(round(Sc[N-1],2))+"g/kg")
print("-----------------------------------------")

#Calculations for diluate stream 
Md=(Ns_d[N-1]*MWs/1000+Nw_d[N-1]*MWw/1000) #mass flow rate (kg/hr)
print("Mass flowrate of diluate stream is "+str(round(Md,2))+" kg/hr")

Sd_f=Sd[N-1]
Cd_na_f=Sd_f/MWs*constants.MW_Na
Cd_cl_f=Sd_f/MWs*constants.MW_cl
dd_out=density_calc(T-273, Sd[N-1])/1000 #density of diluate stream
Qd=Md/dd_out #diluate stream volume flow rate (l/hr)

print("volume flowrate diluate stream is "+str(round(Qd,2))+" l/hr")
print("The total effluent concentration dilute is " + str(round(Sd[N-1],2))+"g/kg")

Sd_out=[Cd_na_f, Cd_cl_f]

for i in range(2,len(Csw)):
    Sd_out.append(Csw[i]*Qed_in/Qd)
print("-----------------------------------------")

#solid mass balance
bal=Qed_in-Md-Mc
bal=(Qed_in*sum(Csw) -Md*(sum(Sd_out))-Mc*Sc_o)/1000
print("Mass balance difference is "+str(round(bal,2)))
error_perc=abs(bal)/(Qed_in*sum(Csw))*100
print("Balance error percentage is "+str(round(error_perc,2))+"%")
print("-----------------------------------------")
#Energy consumption 
#power required 
Ws = 0
for j in range(N):
     Ws += Ij * Acp_tot_j * (Ncp * Vcp + Vel)
print("Power required is "+str(round(Ws/1000,2))+"KW")

#Calculate energy consumption for pumping 
Ppump_ed=(Qed_in_d*1+Qed_in_c*1+Qc*2+Qd*1)/1000/3600*1e5/npump
Eel_t_ed=Ws/1000+Ppump_ed/1000
print("Total energy consumption is "+str(round(Eel_t_ed,2))+"KW")
sec_ed=Eel_t_ed/(Qed_in/1000)
print("Specific energy consumption of Electrodialysis (ED) is "+str(round(sec_ed,2))+"KW/m3 feed")



import unittest
from desalsim.economic_f import econom, revenue
from desalsim.density_calc import density_calc
from desalsim import constants
from desalsim import scaleup

class test_Econom(unittest.TestCase):
    def setUp(self):
        # Define sample input data for initialization
        eq_c = 1500000
        el_conc = 10
        s_conc = 0
        chem1_conc=10
        chem1_pr=0.002
        chem2_conc=0
        chem2_pr=0.002
        cw_conc = 0
        wat_conc = 0
        capex_assumptions=[0.25, 0.2, 0.06, 0.15, 0.2]
        
        prd=10  
        prd_name= "Water" 
        
        # Create an instance of the econom class
        self.econom_calc = econom(eq_c, el_conc, s_conc, chem1_conc, chem1_pr, chem2_conc, chem2_pr, cw_conc, wat_conc)
        self.result_capex = self.econom_calc.capex_calc(capex_assumptions)
        self.revenue_calc = revenue(prd, prd_name)
        
    def test_capex_calc(self):
        capex_result = self.result_capex
        
        # Set CAPEX assumptions 
        capex_assumptions=[0.25, 0.2, 0.06, 0.15, 0.2]

        # Call the capex_calc method
        self.result_capex = self.econom_calc.capex_calc(capex_assumptions)
        
        # Assert the correctness of the output
        expected_capex = 3125700  # Assuming this value based on your calculations
        self.assertAlmostEqual(self.result_capex, expected_capex, delta=expected_capex*0.01)


    def test_opex_calc(self):
        # Access the stored result from test_capex_calc
        capex_result = self.result_capex
        
        # Call the opex_calc method
        result_opex = self.econom_calc.opex_calc(7200, 0.253, 0, 0.000118, 0.001, [0.03, 0.05, 0.15, 0.15, 0.15, 0.03, 0.05, 0.05, 0.67])
        
        # Assert the correctness of the output
        expected_opex = 148541.66  # Assuming this value based on your calculations
        self.assertAlmostEqual(result_opex, expected_opex, delta=expected_opex*0.01)




if __name__ == '__main__':
    unittest.main()



import unittest
from desalsim.edbm_unit_f import EDBMCalc  

class TestEDBMCalc(unittest.TestCase):
    def setUp(self):
        # Define sample input data for initialization
        Qin = 1000
        A = 0.4
        I_d = 400
        F=96485.3 #Coulombs/mol
        N = 50 
        C_s_in = [13.44, 20.725, 1.146, 0, 0, 0.18, 0, 10**(-4.71), 3.01551E-11]
        C_b_in = [0, 0, 0, 0, 0, 0, 0, 10**(-7), 10**(-7)]
        C_a_in = [0, 0, 0, 0, 0, 0, 0, 10**(-7), 10**(-7)]
        T = 20 + 273.15
        self.ph_s = 4.71# Sample pH value for the salt channel
        self.I_ext=A*I_d
        self.JA=3.6*self.I_ext/F
        
        # Create an instance of the EDBMCalc class
        self.edbm_calc = EDBMCalc(Qin, A, I_d, N, C_s_in, C_b_in, C_a_in, T)
        self.results_flowrate=self.edbm_calc.flowrate()
        self.results_mass_flowrate=self.edbm_calc.in_mass_flow_rates(self.ph_s)
        self.results_acid_channel=self.edbm_calc.acid_channel()
        self.results_base_channel=self.edbm_calc.base_channel()
        
    def test_flowrate(self):
        # Call the flowrate method
        self.edbm_calc.flowrate()
        self.results_flowrate=self.edbm_calc.flowrate()
        
        # Check if the flow rates are calculated correctly
        expected_Q1_s_in = self.edbm_calc.Qin / self.edbm_calc.N_trip
        self.assertAlmostEqual(self.edbm_calc.Q1_s_in, expected_Q1_s_in, delta=0.001)
        
        expected_Q1_b_in = self.edbm_calc.Qin / self.edbm_calc.N_trip
        self.assertAlmostEqual(self.edbm_calc.Q1_b_in, expected_Q1_b_in, delta=0.001)
        
        expected_Q1_a_in = self.edbm_calc.Qin / self.edbm_calc.N_trip
        self.assertAlmostEqual(self.edbm_calc.Q1_a_in, expected_Q1_a_in, delta=0.001)


    def test_in_mass_flow_rates(self):
        # Call the in_mass_flow_rates method with a sample value for ph_s        
        results_flowrate=self.results_flowrate
        self.edbm_calc.in_mass_flow_rates(self.ph_s)
        self.results_mass_flowrate=self.edbm_calc.in_mass_flow_rates(self.ph_s)
        
        # Assert the correctness of the output
        # Check if the inlet mass flow rates of each ion are calculated correctly
        for i in range(9):
            # Assuming Ci_s_in, Ci_b_in, and Ci_a_in are properly initialized in setUp
            expected_M_s_in_i = self.edbm_calc.Q1_s_in * self.edbm_calc.Ci_s_in[i] * self.edbm_calc.PM_i[i] / 1000
            self.assertAlmostEqual(self.edbm_calc.M_s_in[i], expected_M_s_in_i, delta=0.001)
            
            expected_M_b_in_i = self.edbm_calc.Q1_b_in * self.edbm_calc.Ci_b_in[i] * self.edbm_calc.PM_i[i] / 1000
            self.assertAlmostEqual(self.edbm_calc.M_b_in[i], expected_M_b_in_i, delta=0.001)
            
            expected_M_a_in_i = self.edbm_calc.Q1_a_in * self.edbm_calc.Ci_a_in[i] * self.edbm_calc.PM_i[i] / 1000
            self.assertAlmostEqual(self.edbm_calc.M_a_in[i], expected_M_a_in_i, delta=0.001)

    def test_acid_channel(self):
        results_flowrate=self.results_flowrate
        results_mass_flowrate=self.edbm_calc.in_mass_flow_rates(self.ph_s)
        # Call the acid_channel method
        self.edbm_calc.acid_channel()
        self.results_acid_channel=self.edbm_calc.acid_channel()
        # Assert the correctness of the output
        # Check if the outlet mass flow rates for each ion are calculated correctly
        expected_M_a_out_i = [0.0, 0.21, 0.0, 0.0, 0.0, 0.0, 0.0, 0.006, 1.9e-09]
        for i in range(9):
            # Assuming M_a_out and Ci_a_out are properly calculated in the method
            #self.edbm_calc.M_a_in[i] + self.edbm_calc.JA * self.edbm_calc.PM_i[i]
            self.assertAlmostEqual(self.edbm_calc.M_a_out[i], expected_M_a_out_i[i], delta=expected_M_a_out_i[i]*0.1)
        
        # Check if the total outlet mass flow rate is calculated correctly
        expected_M_a_out_t = sum(self.edbm_calc.M_a_out)+self.edbm_calc.M_h2o_a_out
        self.assertAlmostEqual(self.edbm_calc.M_a_out_t, expected_M_a_out_t, delta=expected_M_a_out_t*0.01)
        
        # Check if the volumetric outlet flow rate is calculated correctly
        expected_Q1_a_out = self.edbm_calc.M_a_out_t / 1  # Assuming the density is 1 kg/l
        self.assertAlmostEqual(self.edbm_calc.Q1_a_out, expected_Q1_a_out, delta=expected_Q1_a_out*0.01)
        
        # Check if the outlet concentration of single ions in the channel is calculated correctly
        for i in range(9):
            expected_Ci_a_out_i = self.edbm_calc.M_a_out[i] / (self.edbm_calc.Q1_a_out * self.edbm_calc.PM_i[i] / 1000)
            self.assertAlmostEqual(self.edbm_calc.Ci_a_out[i], expected_Ci_a_out_i, delta=0.01)


    def test_base_channel(self):
        results_flowrate=self.results_flowrate
        results_mass_flowrate=self.edbm_calc.in_mass_flow_rates(self.ph_s)
        results_acid_channel=self.edbm_calc.acid_channel()
        # Call the acid_channel method
        self.edbm_calc.base_channel()
        self.results_base_channel=self.edbm_calc.base_channel()
        # Assert the correctness of the output
        # Check if the outlet mass flow rates for each ion are calculated correctly
        expected_M_b_out_i = [0.14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.9e-09, 0.10]
        for i in range(9):
            # Assuming M_b_out and Ci_b_out are properly calculated in the method            
            self.assertAlmostEqual(self.edbm_calc.M_b_out[i], expected_M_b_out_i[i], delta=0.01)
        
        # Check if the total outlet mass flow rate is calculated correctly
        expected_M_b_out_t = sum(self.edbm_calc.M_b_out)+self.edbm_calc.M_h2o_b_out
        self.assertAlmostEqual(self.edbm_calc.M_b_out_t, expected_M_b_out_t, delta=expected_M_b_out_t*0.01)
        
        # Check if the volumetric outlet flow rate is calculated correctly
        expected_Q1_b_out = self.edbm_calc.M_b_out_t / 1  # Assuming the density is 1 kg/l
        self.assertAlmostEqual(self.edbm_calc.Q1_b_out, expected_Q1_b_out, delta=expected_Q1_b_out*0.01)
        
        # Check if the outlet concentration of single ions in the channel is calculated correctly
        for i in range(9):
            expected_Ci_b_out_i = self.edbm_calc.M_b_out[i] / (self.edbm_calc.Q1_b_out * self.edbm_calc.PM_i[i] / 1000)
            self.assertAlmostEqual(self.edbm_calc.Ci_b_out[i], expected_Ci_b_out_i, delta=0.01)

    def test_salt_channel(self):
        results_flowrate=self.results_flowrate
        results_mass_flowrate=self.edbm_calc.in_mass_flow_rates(self.ph_s)
        results_acid_channel=self.edbm_calc.acid_channel()
        results_base_channel=self.edbm_calc.base_channel()
        #Membrane characteristics
        Cm_bp_H= 0.0000001 #mol/l 
        Cm_bp_OH= 0.0000001 #mol/l 
        # Call the salt_channel method
        self.edbm_calc.salt_channel(Cm_bp_H, Cm_bp_OH)
        
        # Assert the correctness of the output
        # Check if the outlet mass flow rates for each ion are calculated correctly
        expected_M_s_out_i = [0.13, 0.20,0.02, 0.0, 0.0, 0.004,0.004, 3.89e-07, 6.03e-13]
        for i in range(9):
            # Assuming M_s_out and Ci_s_out are properly calculated in the method
            self.assertAlmostEqual(self.edbm_calc.M_s_out[i], expected_M_s_out_i[i], delta=0.01)
        
        # Check if the total outlet mass flow rate is calculated correctly
        expected_M_s_out_t = sum(self.edbm_calc.M_s_out)+self.edbm_calc.M_h2o_s_out
        self.assertAlmostEqual(self.edbm_calc.M_s_out_t, expected_M_s_out_t, delta=expected_M_s_out_t*0.01)
        
        # Check if the volumetric outlet flow rate is calculated correctly
        expected_Q1_s_out = self.edbm_calc.M_s_out_t / self.edbm_calc.d_s  # Assuming the density is d_s kg/l
        self.assertAlmostEqual(self.edbm_calc.Q1_s_out, expected_Q1_s_out, delta=expected_Q1_s_out*0.01)
        
        # Check if the outlet concentration of single ions in the channel is calculated correctly
        for i in range(9):
            expected_Ci_s_out_i = self.edbm_calc.M_s_out[i] / (self.edbm_calc.Q1_s_out * self.edbm_calc.PM_i[i] / 1000)
            self.assertAlmostEqual(self.edbm_calc.Ci_s_out[i], expected_Ci_s_out_i, delta=0.01)


if __name__ == '__main__':
    unittest.main()



import math
from desalsim.density_calc import density_calc 
from desalsim import constants 
from desalsim import scaleup
import math
#%%
    #molecular weight
MW_Na=constants.MW_Na
MW_Cl=constants.MW_cl
MW_SO4=constants.MW_so4
MW_K=constants.MW_K
MW_Ca=constants.MW_Ca
MW_Mg=constants.MW_Mg
MW_HCO3=constants.MW_HCO3
MW_values = [MW_Na, MW_Cl, MW_K, MW_Mg, MW_Ca, MW_SO4]
MW_MgOH=constants.MW_MgOH
MW_CaOH=constants.MW_CaOH

#%%
class MFPFRCALC:
    """
    A class used to represent Mass Balance for percipitation step with the addition of alkaline solution 

    Attributes
    ----------
    MW : float
        Molecular weight of the solute (g/mol)
    Ci_in : float
        Initial concentration of the solute (g/L)
    conv_1: float
        Conversion rate for Magnisium percipitation in the first step 
    conv_2: float
        Conversion rate for Calcium percipitation in the second step    
    C_NaOH_1: float
        Concentration of NaOH solution for first step (mol/L)
    Qin
    QMg_in 
    QNaOH_1 : float 
        Volumetric flow rate of sodium hydroxide in step 1 in L/h
    M_MgOH2_1: float 
        Outlet mass flow rate of magnesium hydroxide produced in kg/h
    Qtot_out_1 : float 
        Outlet volumetric flow rate in L/h
    Mtot_out_1 : float 
        Step 1 outlet mass flow rate in kg/h
    magma_d_1 : float 
        magma density: the quantity of solids produced per volume of slurry kg/l
    ph_1 : float 
        ph of solution during first step 
    kps_MgOH : float 
        Product solubility of Mg(OH)2 
    Ci_out_1 : float 
        The outlet ion concentration from step 1 in mol/L
    QNaOH_2_st : float 
        The stoichiometric volumetric flow rate of sodium hydroxide for the second step L/hr
    QNaOH_2_add : float
        The added volumetric flow rate of sodium hydroxide needed to reach a pH = 13  L/h
    M_CaOH2_2: float 
        The outlet mass flow rate of calcium hydroxide produced during the 2nd step kg/hr 
    M_MgOH2_2 : float 
        The outlet mass flow rate of magnesium hydroxide produced during the 2nd step kg/hr 
    magma_d_2 : float    
        Magma density: the quantity of solids produced per volume of slurry kg/l
    Qtot_out_2 : float 
        Total outlet volumetric flow rate for 2nd step in L/h
    Ci_out_2 : float 
        The outlet ion concentration from step 2 in mol/L
      
    Methods
    -------
    calc_step1():
        Calculates the flowrates, the concentration of the streams and the requirements of alkaline solution in the 1st step 
    calc_step2():
        Calculates the flowrates, the concentration of the streams and the requirements of alkaline solution in the 2nd step 
        """
    def __init__(self, Qin, Cin_mfpfr, C_NaOH_1, C_NaOH_2, conv_1, conv_2):
        self.Qin=Qin
        Cc1, Cc2, Cc3, Cc4, Cc5, Cc6=Cin_mfpfr
        self.CNa_in=Cc1/MW_Na #Na concentration (g/l) to (mol/l)
        self.CCl_in=Cc2/MW_Cl #Cl concentration (g/l) to (mol/l)
        self.CK_in=Cc3/MW_K #K concentration (g/l) to (mol/l)
        self.CMg_in=Cc4/MW_Mg #Mg concentration (g/l) to (mol/l)
        self.CCa_in=Cc5/MW_Ca #Ca concentration (g/l) to (mol/l)
        self.CSO4_in=Cc6/MW_SO4 #SO4 concentration (g/l) to (mol/l)
        self.C_NaOH_1=C_NaOH_1 #Molar concentration of NaOH (mol/l) used in the first precipitation step 
        self.C_NaOH_2=C_NaOH_2 #Molar concentration of NaOH (mol/l) used in the first precipitation step 
        self.conv_1=conv_1 #Conversion rate of Mg the first precipitation step 
        self.conv_2=conv_2 #Conversion rate of Mg the first precipitation step 
        self.d_in=density_calc(25, sum(Cin_mfpfr))/1000
    
    def calc_step1(self, kps_MgOH, d_mgoh_2):
        #Calculate the molar flow rate of magnesium in the reactor during the 1° stepin mol/h
        self.QMg_in=self.Qin*self.CMg_in 
        
        #Calculate the volumetric flow rate of sodium hydroxide in L/h
        self.QNaOH_1=(self.Qin*self.CMg_in*(self.conv_1/100)*2)/self.C_NaOH_1 
        
        #Calculate outlet mass flow rate of magnesium hydroxide produced in kg/h
        self.M_MgOH2_1=(self.Qin*self.CMg_in*(self.conv_1/100)*MW_MgOH)/1000 
        
        #Calculate outlet volumetric flow rate in L/h
        self.Qtot_out_1=self.Qin+self.QNaOH_1
        
        #Calculate outlet mass flow rate in kg/h
        self.Mtot_out_1=self.Qin*self.d_in+self.QNaOH_1*1.04-self.M_MgOH2_1  
        
        #Calculate magma density: the quantity of solids produced per volume of slurry kg/l
        self.magma_d_1=self.M_MgOH2_1/self.Qtot_out_1 
        
        #Calculate ph of solution during the first step 
        self.ph_1=14+math.log10(2*(kps_MgOH/4)**(1/3))
        
        #Calculate total outlet volumetric flow rate in L/h
        self.Qtot_out_1=self.Qin+self.QNaOH_1-self.M_MgOH2_1/d_mgoh_2 
        
        #Calculates the outlet ion concentration in mol/L
        self.CMg_out_1=(self.QMg_in*(1-self.conv_1/100))/self.Qtot_out_1 
        self.CNa_out_1=(self.Qin*self.CNa_in+self.QNaOH_1*self.C_NaOH_1)/self.Qtot_out_1
        self.CCl_out_1=(self.Qin*self.CCl_in)/self.Qtot_out_1
        self.CK_out_1=(self.Qin*self.CK_in)/self.Qtot_out_1
        self.CCa_out_1=(self.Qin*self.CCa_in)/self.Qtot_out_1
        self.CSO4_out_1=(self.Qin*self.CSO4_in)/self.Qtot_out_1
        
        return self.QMg_in, self.QNaOH_1, self.M_MgOH2_1, self.Qtot_out_1, self.Mtot_out_1, self.magma_d_1, self.ph_1, self.CMg_out_1, self.CNa_out_1, self.CCl_out_1, self.CK_out_1, self.CCa_out_1, self.CSO4_out_1
    
    def calc_step2(self, d_mgoh_2, d_caoh_2 ):
       #Calculate the molar flow rate of calcium in the reactor during the 2nd stepin mol/h
       self.QCa_in_2=self.Qtot_out_1*self.CCa_out_1 
       
       #Calculate the stoichiometric volumetric flow rate of sodium hydroxide for the second step L/hr
       self.QNaOH_2_st=(self.Qtot_out_1*(self.CCa_out_1*(self.conv_2/100)+self.CMg_out_1*1)*2)/self.C_NaOH_2 
       
       #Calculate concentration of the hydroxide ion in mol/L for a ph=13 solution  
       self.COH_ph13=10**(-(14-13)) 
       self.COH_st=10**(-(14-self.ph_1))

       #Calculate the added volumetric flow rate of sodium hydroxide needed to reach a pH = 13  L/h
       self.QNaOH_2_add=((0.0216-self.COH_ph13)*(self.QNaOH_2_st+self.Qtot_out_1))/(self.COH_ph13-self.C_NaOH_2)
       
       #Calculate the total outlet volumetric flow rate from 2nd step L/h
       self.Qtot_out_2=self.Qtot_out_1+self.QNaOH_2_st+self.QNaOH_2_add
      
       #Calculate the outlet mass flow rate of calcium hydroxide produced during the 2nd step kg/hr 
       self.M_CaOH2_2=(self.Qtot_out_1*self.CCa_out_1*(self.conv_2/100)*MW_CaOH)/1000 #kg/hr
      
       #Calculate the outlet mass flow rate of magnesium hydroxide produced during the 2nd step kg/hr 
       self.M_MgOH2_2=self.Qtot_out_1*self.CMg_out_1*MW_MgOH/1000 
       
       #Calculate magma density: the quantity of solids produced per volume of slurry kg/l
       self.magma_d_2=(self.M_MgOH2_2+self.M_CaOH2_2)/self.Qtot_out_2 
      
       #Calculate total outlet volumetric flow rate for 2nd step in L/h
       self.Qout_2=self.Qtot_out_2-self.M_CaOH2_2/d_caoh_2-self.M_MgOH2_2/d_mgoh_2
      
       
       #Calculates the outlet ion concentration in mol/L
       self.CNa_out_2=(self.Qtot_out_1*self.CNa_out_1+(self.QNaOH_2_st+self.QNaOH_2_add)*self.C_NaOH_2)/self.Qtot_out_2       
       self.CCa_out_2=(self.QCa_in_2*(1-self.conv_2/100))/self.Qtot_out_2
       self.CCl_out_2=(self.Qtot_out_1*self.CCl_out_1)/self.Qtot_out_2
       self.CK_out_2=(self.Qtot_out_1*self.CK_out_1)/self.Qtot_out_2
       self.CMg_out_2=0
       self.CSO4_out_2=(self.Qtot_out_1*self.CSO4_out_1)/self.Qtot_out_2
       
       #Calculate ph of solution during the second step 
       self.ph_2=14+math.log10(0.1)
    
       return self.QCa_in_2, self.QNaOH_2_st, self.COH_ph13, self.QNaOH_2_add, self.Qtot_out_2, self.M_CaOH2_2, self.M_MgOH2_2, self.magma_d_2, self.Qout_2, self.CNa_out_2, self.CCa_out_2, self.CCl_out_2, self.CK_out_2, self.CMg_out_2, self.CSO4_out_2,  self.ph_2

class HClAddition:
    def __init__(self, Qout_2, Cout_all_m, MW_cl, ph_2, HCl_conc):
        self.Qout_2 = Qout_2
        self.Cout_all_m = Cout_all_m
        self.MW_cl = MW_cl
        self.ph_2 = ph_2
        self.HCl_conc=HCl_conc

    def calculate_HCl_addition(self, Cout_mfpfr_g):
        OH_initial = math.pow(10, -14) / math.pow(10, -self.ph_2)
        OH_final = math.pow(10, -14) / math.pow(10, -7)
        
        #Calculate the volume of HCl 1M in liters, Qout_f
        QHCl = (self.Qout_2 * OH_initial - self.Qout_2 * OH_final) / (OH_final + self.HCl_conc)  
        self.Qout_f = self.Qout_2+QHCl

        # Calculate the outlet concentration of chloride ions
        C_cl_out = (self.Cout_all_m[1] * self.Qout_2 + QHCl * self.HCl_conc) / self.Qout_f  # mol/l
        self.Cout_all_m[1] = C_cl_out
        C_cl_out = C_cl_out * self.MW_cl  # g/l
        Cout_mfpfr_g[1] = C_cl_out

        for i in range(2, 6):
            Cout_mfpfr_g[i] = Cout_mfpfr_g[i] * self.Qout_2 / self.Qout_f

        Cout_mfpfr_g[0] = Cout_mfpfr_g[0] * self.Qout_2 / self.Qout_f
        # Return the volume of HCl added and the outlet concentration of chloride ions
        return QHCl, Cout_mfpfr_g
     
  # Get the outlet flow rate

#%%Energy consumption 
class energycons:
    def energycalc(Qtot, QNaOH, Qin, QNaOH_1, QNaOH_2_add, QNaOH_2_st, dp, npump):
        Qtot=Qtot
        QNaOH=QNaOH
        Qin=Qin
        QNaOH_2_st=QNaOH_2_st
        QNaOH_2_add=QNaOH_2_add
        QNaOH_1=QNaOH_1        
        
        #Calculate the energy required for pumping in kW
        Epump_1=(Qin*dp+QNaOH_1*dp)*1e5/3600/(1000*npump) #(W)
        Epump_2=((Qin+QNaOH_1)*dp+(QNaOH_2_add+QNaOH_2_st)*dp)*1e5/3600/(1000*npump)       #(W)
        return Epump_1, Epump_2
                    
#%%
#Example usage
#Input parameters
    #molecular weight
MW_Na=constants.MW_Na
MW_Cl=constants.MW_cl
MW_SO4=constants.MW_so4
MW_K=constants.MW_K
MW_Ca=constants.MW_Ca
MW_Mg=constants.MW_Mg
MW_HCO3=constants.MW_HCO3
MW_values = [MW_Na, MW_Cl, MW_K, MW_Mg, MW_Ca, MW_SO4]
MW_MgOH=constants.MW_MgOH
MW_CaOH=constants.MW_CaOH

#constants
kps_MgOH=5.61*0.000000000001 #product solublity of Mg(OH)2
kps_CaOH=5.5*0.000001 #product solublity of Ca(OH)2
d_mgoh_2=2.34 #Mg(OH)2 density (units: kg/l)
d_caoh_2=2.211 #Ca(OH)2 density (units: kg/l)

#assumptions
dp=0.5 # pressure drop (units: bar)
dp_HCl=0.3# pressure drop HCl solution (units: bar)
npump=0.8 #pump efficiency (units: -)


# Define the input parameters
T=20+273.15 #Temperature (units: K)
C_NaOH = [1, 1]  # Concentration of NaOH solution for step 1 and step 2 in MOL/L
conv = [95, 93]  # Conversion rate for step 1 and step 2
Cin_mfpfr = [17.3, 38.6, 0.6, 4.3, 1.2, 9.9]  # Concentrations of [Na, Cl, K, Mg, Ca, SO4]
Qin_mfpfr = 1000  # Flow rate in l/hr

# Calculate the density of the input
d_in = density_calc(25, sum(Cin_mfpfr)) / 1000

# Create an instance of the inputpar class with the defined parameters
mfpfr_dat = MFPFRCALC(Qin_mfpfr, Cin_mfpfr, *C_NaOH, *conv)

# Call the calc_step1 and calc_step2 methods to calculate the necessary values
mfpfr_dat.calc_step1(kps_MgOH, d_mgoh_2)
mfpfr_dat.calc_step2(d_mgoh_2, d_caoh_2 )
ph_2=mfpfr_dat.ph_2

# Calculate the total outlet concentration and the concentration of sulfate ions
Cour_mfpfr = sum([mfpfr_dat.CNa_out_2, mfpfr_dat.CCa_out_2, mfpfr_dat.CCl_out_2, mfpfr_dat.CK_out_2, mfpfr_dat.CMg_out_2, mfpfr_dat.CSO4_out_2]) # mol/l
CSO4_out_2 = mfpfr_dat.CSO4_out_2 # mol/l

# Calculate the concentration of sodium chloride ions
CNa_out_2 = mfpfr_dat.CNa_out_2
Cnacl_out = CNa_out_2 - 2 * CSO4_out_2

# Get the molar masses of the compounds
M_MgOH2_1 = mfpfr_dat.M_MgOH2_1
M_CaOH2 = mfpfr_dat.M_CaOH2_2
M_MgOH2 = mfpfr_dat.M_MgOH2_2

# Create a list of the outlet concentrations in mol/l
Cout_all_m = [mfpfr_dat.CNa_out_2, mfpfr_dat.CCl_out_2, mfpfr_dat.CK_out_2, mfpfr_dat.CMg_out_2, mfpfr_dat.CCa_out_2, mfpfr_dat.CSO4_out_2]

# Calculate the outlet concentrations in g/l
MW = [MW_Na, MW_Ca, MW_Cl, MW_K, MW_Mg, MW_SO4]
Cout_mfpfr_g = [Cout_all_m[i] * MW[i] for i in range(len(Cout_all_m))] # g/l

#Outlet flow rate 
Qout_2 = mfpfr_dat.Qout_2

# Calculate the chemical consumption of NaOH
QNAOH = mfpfr_dat.QNaOH_1 + mfpfr_dat.QNaOH_2_add + mfpfr_dat.QNaOH_2_st # convert to kg

#HCl to decrease ph to 7
HCl_conc=1 #l of HCl 1M
unit = HClAddition(Qout_2, Cout_all_m, MW_Cl, ph_2, HCl_conc)

# Call the calculate_HCl_addition method
QHCl, Cout_mfpfr_g = unit.calculate_HCl_addition(Cout_mfpfr_g)

# Print the volume of HCl added and the outlet concentration of chloride ions
print(f"HCl flow rate is {round(QHCl,2)} l/hr")
print(f"NaOH flow rate is {round(QNAOH,2)} l/hr")
print("-----------------------------------------")

#Calculate final outlet flow rate
Qout_f=Qout_2+QHCl #l/h
d_out_s=density_calc(25,round(sum(Cout_mfpfr_g),2))/1000 #kg/m3
Mout_f=Qout_f*d_out_s #kg/h

print("Mg(OH)2 mass flow rate is "+str(round(M_MgOH2_1,2))+"kg/hr")
print("Ca(OH)2 mass flow rate is "+str(round(M_CaOH2,2))+"kg/hr")
print("Total effluent flow rate is "+str(round(Mout_f,2))+"kg/hr")
print("Total effluent flow rate is "+str(round(Qout_f,2))+"kg/hr")
print("-----------------------------------------")

#Mass balance 
bal=Qin_mfpfr *d_in+QNAOH*1.04+QHCl*1.01-M_MgOH2_1-M_CaOH2-Mout_f-M_MgOH2
error_perc=abs(bal)/(Qin_mfpfr *d_in+QNAOH+QHCl)*100

print("Mass balance difference is "+str(round(bal,2)))
print("Balance error percentage is "+str(round(error_perc,2))+"%")
print("-----------------------------------------")

#electicity consumption
    # Create an instance of the inputpar class with the defined parameters
Epump_1, Epump_2=energycons.energycalc(mfpfr_dat.Qout_2, QNAOH, Qin_mfpfr, mfpfr_dat.QNaOH_1, mfpfr_dat.QNaOH_2_add, mfpfr_dat.QNaOH_2_st, dp, npump)

    #Electricity consumption for pumping , KWh
E_el_mfpf=(Epump_1+Epump_2+(QHCl*dp_HCl)*1e5/3600/(1000*npump))/1000
print("Total electricity energy consumption is "+str(round(E_el_mfpf,2))+ " KW")

    #Electricity consumption for filtration unit 
E_fil=scaleup.scaleup(0.5, 0.3*1000, Mout_f)

    #Total electricity consumption, KWh
E_el_mfpf=E_el_mfpf+E_fil

    #Specific energy consumption per kg of Mg(OH)2, KWh/kg of Mg(OH)2
SEC_el_prod=(E_el_mfpf)/(M_MgOH2)
print("Specific energy consumption per product is "+str(round(SEC_el_prod,2))+" KWh/kg product ")

    #Specific energy consumption per feed, KWh/m3 of feed
SEC_el_feed=(E_el_mfpf)/(Qin_mfpfr/1000)
print("Specific energy consumption per brine intake is "+str(round(SEC_el_feed,2))+" KWh/m3 of feed ")



import unittest
from desalsim import constants
from desalsim.ed_unit_f import ElectrodialysisCalc

# Define the input parameters for testing
C_feed = [10.36, 15.39, 0.36, 0.028, 0.02, 0.07]  # Feed concentrations in g/L
Q_feed = 1000  # Feed flow rate in L/hr
T_feed = 40  # Feed temperature in °C
T_dial = 25  # Dialysate temperature in °C
A_membrane = 10  # Membrane area in m^2
V_cell = 100  # Voltage across the cell in V
I_current = 50  # Current passing through the cell in A

class test_ElectrodialysisCalc(unittest.TestCase):
    def setUp(self):        
        # Create an instance of the ElectrodialysisCalc class with the defined parameters
        self.ed_calc = ElectrodialysisCalc()
        
    def test_Ts_cp(self):
        # Call the Ts_cp method
        result = ElectrodialysisCalc.Ts_cp(10)
        
        # Define the expected value
        expected_result = 0.9676  # Expected result for S=10
        
        # Assert the equality of the calculated value and expected value
        self.assertAlmostEqual(result, expected_result, delta=expected_result*0.01)

    def test_Tw_cp(self):
        # Call the Tw_cp method
        Sc=43
        Sd=43
        result = ElectrodialysisCalc.Tw_cp(Sc, Sd)
        
        # Define the expected value
        expected_result = 10.3  
        
        # Assert the equality of the calculated value and expected value
        self.assertAlmostEqual(result, expected_result, delta=expected_result*0.01)
        
    def test_Lw_cp(self):
        # Call the Lw_cp method
        S = 43
        result = ElectrodialysisCalc.Lw_cp(S)
        
        # Define the expected value
        expected_result = 5 * S ** (-0.416)
        
        # Assert the equality of the calculated value and expected value
        self.assertAlmostEqual(result, expected_result, delta=expected_result*0.01)

    def test_p_osmo(self):
        # Call the p_osmo method
        S = 43
        T = 25
        MWs = constants.MW_NaCl
        
        result = ElectrodialysisCalc.p_osmo(S, T, MWs)
        
        # Define the expected value
        C1 = S / MWs * constants.MW_Na / constants.MW_Na
        C2 = S / MWs * constants.MW_cl / constants.MW_cl
        sum_Ci = C1 + C2
        expected_result = 0.0831446261815324 * sum_Ci * T
        
        # Assert the equality of the calculated value and expected value
        self.assertAlmostEqual(result, expected_result, delta=expected_result*0.01)

    def test_dC(self):
        # Define parameters
        Ts_cp = 0.95
        tcu = 0.5
        D = 1.61e-9 #Diffusion coefficient (m^2/s)
        Ij = 50
        h = 0.5
        Sh = 18

        
        # Call the dC method
        result = ElectrodialysisCalc.dC(Ts_cp, tcu, D, Ij, h, Sh)
        
        # Define the expected value
        F = 96485.3329  # Faraday constant (C/mol)
        Tcu = (Ts_cp + 1) / 2
        expected_result = -(Tcu - tcu) / D * (Ij / F) * (2 * h / 1000 / Sh)
        
        # Assert the equality of the calculated value and expected value
        self.assertAlmostEqual(result, expected_result, delta=expected_result*0.01)


if __name__ == '__main__':
    unittest.main()



#scenarios comparison 

#Import results 
import example_1 as sc1
import example_2 as sc2

#import 
import numpy as np
import pandas as pd
import numpy as np 
import matplotlib.pyplot as plt 
from pandas.plotting import register_matplotlib_converters
import matplotlib.ticker as ticker
register_matplotlib_converters()

hr=300*24 #hours   

#%%figures 

X = ['Example 1', 'Example 2']
X_axis = np.arange(len(X))

#Figure 1: Electrical consumption Vs thermal consumption 
Eel = [ sc1.sum_el_en, sc2.sum_el_en]
Eth = [sc1.sum_th_en,   sc2.sum_th_en]

Eel = [i * hr/1e6 for i in Eel]
Eth = [i * hr/1e6 for i in Eth]
plt.bar(X_axis - 0.2, Eel, 0.4, color="#00516a", label = 'Electrical (GWel)')
plt.bar(X_axis + 0.2, Eth, 0.4, color="sandybrown", label = 'Thermal (GWth)')
plt.xticks(X_axis, X)
plt.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
plt.xlabel("Scenarios")
plt.ylabel("Eenergy consumption (GW)")
plt.legend()
plt.savefig('electricVSthermal.png')
plt.show()

#Figure 2: water production in kg/hr
Qw = [sc1.Qw_tot, sc2.Qw_tot]

Qw_y = [i * hr/1E6 for i in Qw]
plt.bar(X_axis - 0.4, Qw_y, 0.4, color="#00516a")  
plt.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
plt.xticks(X_axis, X)
plt.xlabel("Scenarios")
plt.ylabel("Water production in 1000 m\u00B3/year")
plt.savefig('water_production.png')
plt.show()

#Figure 3: Capex 
Cap = [sc1.CAPEX, sc2.CAPEX]
Cap = [i/1e6 for i in Cap]
plt.bar(X_axis - 0.4, Cap, 0.4, color="#00516a")  
plt.xticks(X_axis, X)
plt.xlabel("Scenarios")
plt.ylabel("Capex (M€)")
plt.savefig('capex.png')
plt.show()

#Figure 4: OPEX 
OPEX = [sc1.OPEX, sc2.OPEX]
OPEX = [i/1e6 for i in OPEX]
plt.bar(X_axis - 0.4, OPEX, 0.4, color="#00516a")    
plt.xticks(X_axis, X)
plt.xlabel("Scenarios")
plt.ylabel("OPEX (M€/year)")
plt.savefig('OPEX.png')
plt.show()

#Figure 5: Revenues 
Rev= [sc1.reve_t,  sc2.reve_t]
Rev = [i/1e6 for i in Rev]
plt.bar(X_axis - 0.4, Rev, 0.4, color="#00516a")  
plt.xticks(X_axis, X)
plt.xlabel("Scenarios")
plt.ylabel("Revenues (M€/year)")
plt.savefig('revenues.png')
plt.show()

#Figure 6: CO2 emissions 
co2_em=[sc1.emis_t, sc2.emis_t]
co2_em = [i * hr/1e6 for i in co2_em]
plt.bar(X_axis - 0.4, co2_em, 0.4, color="#00516a")  
plt.xticks(X_axis, X)
plt.xlabel("Scenarios")
plt.ylabel("CO\u2082 emissions (KTon/year)")
plt.savefig('CO2emissions.png')
plt.show()


#%% Create dataframes 
ind=np.array(["Water production", "Total electrical consumption (GWh)", "Total thermal energy consumption (GWh)","Carbon dioxide emission (Kton co2/year) ",
              "OPEX (M€/year)", "CAPEX (M€)"])
sum_res_1=np.array([Qw_y[0],Eel[0], Eth[0],co2_em[0], OPEX[0], Cap[0]])
df1=pd.DataFrame(sum_res_1, ind)

sum_res_2=np.array([Qw_y[1],Eel[1], Eth[1],co2_em[1], OPEX[1], Cap[1]])
df2=pd.DataFrame(sum_res_2, ind)

#all
df=pd.DataFrame({'sce 1': [Qw_y[0],Eel[0], Eth[0],co2_em[0], OPEX[0], Cap[0]], 
                 'sce 2': [Qw_y[1],Eel[1], Eth[1],co2_em[1], OPEX[1], Cap[1]],
                 })

sc1_dfel=pd.DataFrame(sc1.E_el_all , sc1.tec_names)
sc1_dfeth=pd.DataFrame(sc1.E_th_all, sc1.tec_names)
sc2_dfel=pd.DataFrame(sc2.E_el_all , sc2.tec_names)
sc2_dfeth=pd.DataFrame(sc2.E_th_all, sc2.tec_names)

#%% Export to excel
with pd.ExcelWriter('results_scenario.xlsx') as writer:
    df1.to_excel(writer,sheet_name="example 1")
    df2.to_excel(writer,sheet_name="example 2")
    df.to_excel(writer,sheet_name="all scenarios")
    
    sc1.sum_table_C.to_excel(writer,sheet_name="example 1", startcol=0, startrow=20, header=False)
    sc1.sum_table_f.to_excel(writer,sheet_name="example 1", startcol=0, startrow=18, header=True, index=True )
    sc1.sum_table_d.to_excel(writer,sheet_name="example 1", startcol=0, startrow=29, header=True)

    sc2.sum_table_C.to_excel(writer,sheet_name="example 2", startcol=0, startrow=20, header=False)
    sc2.sum_table_f.to_excel(writer,sheet_name="example 2", startcol=0, startrow=18, header=True)
    sc2.sum_table_d.to_excel(writer,sheet_name="example 2", startcol=0, startrow=30, header=False)
    
    sc2_dfel.to_excel(writer,sheet_name="example 2", startcol=0, startrow=33, header=True)
    sc2_dfeth.to_excel(writer,sheet_name="example 2", startcol=2, startrow=33, header=True)
    
    sc1_dfel.to_excel(writer,sheet_name="example 1", startcol=0, startrow=33, header=True)
    sc1_dfeth.to_excel(writer,sheet_name="example 1", startcol=2, startrow=33, header=True)


    